@@ -1,6 +1,6 @@
 ;;;; batch-driver.scm - Driver procedure for the compiler
 ;
-; Copyright (c) 2008-2021, The CHICKEN Team
+; Copyright (c) 2008-2022, The CHICKEN Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -27,7 +27,7 @@
 
 (declare
   (unit batch-driver)
-  (uses extras data-structures pathname
+  (uses extras data-structures pathname expand
 	support compiler-syntax compiler optimizer internal
 	;; TODO: Backend should be configurable
 	scrutinizer lfa2 c-platform c-backend user-pass))
@@ -47,7 +47,10 @@
 	chicken.pretty-print
 	chicken.process-context
 	chicken.string
+	chicken.syntax
+        chicken.port
 	chicken.time
+        chicken.condition
 	chicken.compiler.support
 	chicken.compiler.compiler-syntax
 	chicken.compiler.core
@@ -216,7 +219,7 @@
 			      '()
 			      `((import-syntax ,@default-imports)))))
 	(cleanup-forms '(((chicken.base#implicit-exit-handler))))
-	(outfile (cond ((memq 'output-file options) 
+	(outfile (cond ((memq 'output-file options)
 			=> (lambda (node)
 			     (let ((oname (option-arg node)))
 			       (if (symbol? oname)
@@ -224,18 +227,15 @@
 				   oname) ) ) )
 		       ((memq 'to-stdout options) #f)
 		       (else (make-pathname #f (if filename (pathname-file filename) "out") "c")) ) )
-	(ipath (map chop-separator
-		    (##sys#split-path
-		     (or (get-environment-variable "CHICKEN_INCLUDE_PATH") "")))) 
 	(opasses (default-optimization-passes))
 	(time0 #f)
 	(time-breakdown #f)
 	(forms '())
 	(inline-output-file #f)
 	(profile (or (memq 'profile options)
-		     (memq 'accumulate-profile options) 
+		     (memq 'accumulate-profile options)
 		     (memq 'profile-name options)))
-	(profile-name 
+	(profile-name
 	 (and-let* ((pn (memq 'profile-name options))) (cadr pn)))
 	(hsize (memq 'heap-size options))
 	(kwstyle (memq 'keyword-style options))
@@ -282,6 +282,25 @@
 	   (newline))
 	 xs) ) )
 
+    (define (string-trim str)
+      (let loop ((front 0)
+		 (back (string-length str)))
+	(cond ((= front back) "")
+	      ((char-whitespace? (string-ref str front))
+	       (loop (add1 front) back))
+	      ((char-whitespace? (string-ref str (sub1 back)))
+	       (loop front (sub1 back)))
+	      (else (substring str front back)))))
+       
+    (define (string->extension-name str)
+      (let ((str (string-trim str)))
+        (if (and (positive? (string-length str))
+                 (char=? #\( (string-ref str 0)))
+            (handle-exceptions ex 
+              (##sys#error "invalid import specification" str)
+              (with-input-from-string str read))
+            (string->symbol str))))
+
     (define (arg-val str)
       (let* ((len (string-length str))
 	     (len1 (- len 1)) )
@@ -317,6 +336,13 @@
 		  no contf) )
 	  db) ) )
 
+    (define (chop-separator str)
+      (let ((len (sub1 (string-length str))))
+        (if (and (> len 0)
+                 (memq (string-ref str len) '(#\\ #\/)))
+            (substring str 0 len)
+            str) ) )
+
     (when unit
       (set! unit-name (string->symbol (option-arg unit))))
     (when (or unit-name dynamic)
@@ -325,7 +351,7 @@
       (set! ##sys#dload-disabled #t)
       (repository-path #f))
     (set! enable-specialization (memq 'specialize options))
-    (set! debugging-chicken 
+    (set! debugging-chicken
       (append-map
        (lambda (do)
 	 (map (lambda (c) (string->symbol (string c)))
@@ -430,8 +456,7 @@
     (set! ##sys#read-error-with-line-number #t)
     (set! ##sys#include-pathnames
       (append (map chop-separator (collect-options 'include-path))
-	      ##sys#include-pathnames
-	      ipath) )
+	      ##sys#include-pathnames) )
     (when (and outfile filename (string=? outfile filename))
       (quit-compiling "source- and output-filename are the same") )
     (when (memq 'keep-shadowed-macros options)
@@ -497,7 +522,7 @@
     (set! import-forms
       (append
        import-forms
-       (map (lambda (r) `(import ,(string->symbol r)))
+       (map (lambda (r) `(import ,(string->extension-name r)))
 	    (collect-options 'require-extension))))
 
     (when (memq 'compile-syntax options)
@@ -587,7 +612,8 @@
 			       (in (check-and-open-input-file f)) )
 			  (fluid-let ((##sys#current-source-filename f))
 			    (let loop ()
-			      (let ((x (read/source-info in)))
+			      (let ((x (chicken.syntax#read-with-source-info in))) ; OBSOLETE - after bootstrapping we can get rid of this explicit namespacing
+
 				(cond ((eof-object? x) 
 				       (close-checked-input-file in f) )
 				      (else
@@ -649,7 +675,7 @@
    	     (when (debugging '|N| "real name table:")
 	       (display-real-name-table) )
 	     (when (debugging 'n "line number database:")
-	       (display-line-number-database) )
+	       (##sys#display-line-number-database) )
 
 	     (set! ##sys#line-number-database line-number-database-2)
 	     (set! line-number-database-2 #f)
@@ -676,7 +702,7 @@
 	       (initialize-analysis-database)
 
 	       ;; collect requirements and load inline files
-	       (let ((extensions required-libraries))
+	       (let ((extensions required-extensions))
 		 (when enable-inline-files
 		   (for-each
 		    (lambda (id)
@@ -851,7 +877,7 @@
 
                               ;; generate link file
 			      (when emit-link-file
-				(let ((exts required-libraries))
+				(let ((exts required-extensions))
 				  (dribble "generating link file `~a' ..." emit-link-file)
 				  (with-output-to-file emit-link-file (cut pp exts))))
 
