@@ -1,6 +1,6 @@
 ;;;; c-platform.scm - Platform specific parameters and definitions
 ;
-; Copyright (c) 2008-2021, The CHICKEN Team
+; Copyright (c) 2008-2022, The CHICKEN Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -149,7 +149,7 @@
 
 (define-constant +flonum-bindings+
   (map (lambda (x) (symbol-append 'chicken.flonum# x))
-       '(fp/? fp+ fp- fp* fp/ fp> fp< fp= fp>= fp<= fpmin fpmax fpneg fpgcd
+       '(fp/? fp+ fp- fp* fp/ fp> fp< fp= fp>= fp<= fpmin fpmax fpneg fpgcd fp*+
 	 fpfloor fpceiling fptruncate fpround fpsin fpcos fptan fpasin fpacos
 	 fpatan fpatan2 fpexp fpexpt fplog fpsqrt fpabs fpinteger?)))
 
@@ -173,6 +173,8 @@
     chicken.base#equal=? chicken.base#exact-integer?
     chicken.base#flush-output
 
+    chicken.base#weak-cons chicken.base#weak-pair? chicken.base#bwp-object?
+
     chicken.base#identity chicken.base#o chicken.base#atom?
     chicken.base#alist-ref chicken.base#rassoc
 
@@ -238,6 +240,7 @@
 
     chicken.locative#locative-ref chicken.locative#locative-set!
     chicken.locative#locative->object chicken.locative#locative?
+    chicken.locative#locative-index
 
     chicken.memory#pointer+ chicken.memory#pointer=?
     chicken.memory#address->pointer chicken.memory#pointer->address
@@ -538,6 +541,7 @@
 (rewrite 'srfi-4#f64vector? 2 1 "C_i_f64vectorp" #t)
 (rewrite 'scheme#pair? 2 1 "C_i_pairp" #t)
 (rewrite '##sys#pair? 2 1 "C_i_pairp" #t)
+(rewrite 'chicken.base#weak-pair? 2 1 "C_i_weak_pairp" #t)
 (rewrite 'scheme#procedure? 2 1 "C_i_closurep" #t)
 (rewrite 'chicken.base#port? 2 1 "C_i_portp" #t)
 (rewrite 'scheme#boolean? 2 1 "C_booleanp" #t)
@@ -565,6 +569,7 @@
 (rewrite 'scheme#inexact? 2 1 "C_u_i_inexactp" #f)
 (rewrite 'scheme#list? 2 1 "C_i_listp" #t)
 (rewrite 'scheme#eof-object? 2 1 "C_eofp" #t)
+(rewrite 'chicken.base#bwp-object? 2 1 "C_bwpp" #t)
 (rewrite 'scheme#string-ref 2 2 "C_subchar" #f)
 (rewrite 'scheme#string-ref 2 2 "C_i_string_ref" #t)
 (rewrite 'scheme#string-set! 2 3 "C_setsubchar" #f)
@@ -652,6 +657,7 @@
 (rewrite 'chicken.flonum#fp/? 16 2 "C_a_i_flonum_quotient_checked" #f words-per-flonum)
 (rewrite 'chicken.flonum#fpneg 16 1 "C_a_i_flonum_negate" #f words-per-flonum)
 (rewrite 'chicken.flonum#fpgcd 16 2 "C_a_i_flonum_gcd" #f words-per-flonum)
+(rewrite 'chicken.flonum#fp*+ 16 3 "C_a_i_flonum_multiply_add" #f words-per-flonum)
 
 (rewrite 'scheme#zero? 5 "C_eqp" 0 'fixnum)
 (rewrite 'scheme#zero? 2 1 "C_u_i_zerop2" #f)
@@ -947,6 +953,7 @@
 
 (rewrite 'scheme#cons 16 2 "C_a_i_cons" #t 3)
 (rewrite '##sys#cons 16 2 "C_a_i_cons" #t 3)
+(rewrite 'chicken.base#weak-cons 16 2 "C_a_i_weak_cons" #t 3)
 (rewrite 'scheme#list 16 #f "C_a_i_list" #t '(0 3) #t)
 (rewrite '##sys#list 16 #f "C_a_i_list" #t '(0 3))
 (rewrite 'scheme#vector 16 #f "C_a_i_vector" #t #t #t)
@@ -1057,6 +1064,8 @@
 (rewrite '##sys#null-pointer? 17 1 "C_null_pointerp" "C_null_pointerp")
 (rewrite '##sys#immediate? 17 1 "C_immp")
 (rewrite 'chicken.locative#locative->object 17 1 "C_i_locative_to_object")
+(rewrite 'chicken.locative#locative->object 17 1 "C_i_locative_to_object")
+(rewrite 'chicken.locative#locative-index 17 1 "C_i_locative_index")
 (rewrite 'chicken.locative#locative-set! 17 2 "C_i_locative_set")
 (rewrite '##sys#foreign-fixnum-argument 17 1 "C_i_foreign_fixnum_argumentp")
 (rewrite '##sys#foreign-char-argument 17 1 "C_i_foreign_char_argumentp")
