@@ -1,6 +1,6 @@
 ;;;; chicken-install.scm
 ;
-; Copyright (c) 2008-2021, The CHICKEN Team
+; Copyright (c) 2008-2022, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -417,16 +417,27 @@
   
 (define (locate-egg name version)
   (let* ((cached (make-pathname cache-directory name))
+         (metadata-dir (make-pathname cache-metadata-directory name))
          (now (current-seconds))
-         (status (make-pathname cached +status-file+))
+         (status (make-pathname metadata-dir +status-file+))
          (eggfile (make-pathname cached name +egg-extension+)))
     (define (fetch lax)
       (when (file-exists? cached)
         (delete-directory cached #t))
+      (when (file-exists? metadata-dir)
+        (delete-directory metadata-dir #t))
       (create-directory cached #t)
-      (fetch-egg-sources name version cached lax)
-      (with-output-to-file status (cut write current-status)))
-    (cond ((or (not (probe-dir cached))
+      (create-directory metadata-dir #t)
+      (fetch-egg-sources name version cached lax))
+    (cond ((and (probe-dir cached)
+                (not (file-exists? status)))
+           ;; If for whatever reason the status file doesn't exist
+           ;; (e.g., it was renamed, as in 2f6a7221), reset the cache
+           ;; of the egg to prevent the object files in there from
+           ;; being reused.
+           (d "resetting ~a, as ~a does not exist~%" cached status)
+           (fetch #f))
+	  ((or (not (probe-dir cached))
                (not (file-exists? eggfile)))
            (d "~a not cached~%" name)
            (when cached-only (error "extension not cached" name))
@@ -441,18 +452,17 @@
                        (error "cached egg does not match CHICKEN version - use `-force' to install anyway" name)))
                  (else (fetch #f)))))
     (let* ((info (validate-egg-info (load-egg-info eggfile)))
-           (vfile (make-pathname cached +version-file+))
-           (tfile (make-pathname cached +timestamp-file+))
+           (vfile (make-pathname metadata-dir +version-file+))
+           (tfile (make-pathname metadata-dir +timestamp-file+))
            (lversion (or (get-egg-property info 'version)
                          (and (file-exists? vfile)
                               (with-input-from-file vfile read)))))
       (cond ((and (not cached-only)
-                  (or (and (string? version)
-                           (not (equal? version lversion)))
-                      (and (or (not (file-exists? tfile))
-                               (> (- now (with-input-from-file tfile read))
-                                  +one-hour+))
-                           (not (check-remote-version name lversion cached)))))
+                  (if (string? version)
+                      (not (equal? version lversion))
+                      (or (and (file-exists? tfile)
+                               (> (- now (with-input-from-file tfile read)) +one-hour+))
+                          (not (check-remote-version name lversion cached)))))
              (d "version of ~a out of date~%" name)
              (fetch #t)
              (let* ((info (validate-egg-info (load-egg-info eggfile))) ; new egg info (fetched)
@@ -470,6 +480,43 @@
              (resolve-location new))))
         (else name)))
 
+(define (locate-local-egg-dir location egg-name version)
+  ;; Locate the directory of egg-name, considering the following
+  ;; directory layouts in order:
+  ;; * <location>/<egg-name>/<egg-name>.egg
+  ;; * <location>/<egg-name>/<version>/<egg-name>.egg
+  ;;
+  ;; Return (values <egg-dir> <version>).  <egg-dir> and <version>
+  ;; will be #f in case they cannot be determined.
+  (let ((egg-dir (probe-dir (make-pathname location egg-name))))
+    (cond
+     ((not egg-dir)
+      (values #f #f))
+     ;; <location>/<egg-name>/<egg-name>.egg
+     ((file-exists? (make-pathname egg-dir egg-name +egg-extension+))
+      (values egg-dir #f))
+     (else
+      ;; <location>/<egg-name>/<version>/<egg-name>.egg
+      (if version
+          (values (probe-dir (make-pathname egg-dir (->string version)))
+                  version)
+          (let ((versions (directory egg-dir)))
+            (if (null? versions)
+                (values #f #f)
+                (let ((latest (car (sort versions version>=?))))
+                  (values (make-pathname egg-dir (->string latest))
+                          latest)))))))))
+
+(define (write-cache-metadata egg egg-version)
+  (let ((metadata-dir (make-pathname cache-metadata-directory egg)))
+    (when egg-version
+      (with-output-to-file (make-pathname metadata-dir +version-file+)
+        (cut write egg-version)))
+    (with-output-to-file (make-pathname metadata-dir +timestamp-file+)
+      (cut write (current-seconds)))
+    (with-output-to-file (make-pathname metadata-dir +status-file+)
+      (cut write current-status))))
+
 (define (fetch-egg-sources name version dest lax)
   (print "fetching " name)
   (let loop ((locs default-locations))
@@ -496,26 +543,32 @@
                        (cond (dir
                                (copy-egg-sources tmpdir dest)
                                (delete-directory tmpdir #t)
-                               (when ver
-                                 (with-output-to-file
-                                   (make-pathname dest +version-file+)
-                                     (cut write ver)))
-                               (with-output-to-file
-                                 (make-pathname dest +timestamp-file+)
-                                 (cut write (current-seconds))))
+			       (write-cache-metadata name ver))
                              (else (loop (cdr srvs))))))))))
-          ((probe-dir (make-pathname (car locs) name))
-           => (lambda (dir)
-                (d "trying location ~a ...~%" dir)
-                (let* ((eggfile (make-pathname dir name +egg-extension+))
-                       (info (validate-egg-info (load-egg-info eggfile)))
-                       (rversion (get-egg-property info 'version)))
-                  (if (or (not rversion)
-                          (not version)
-                          (version>=? rversion version))
-                      (copy-egg-sources dir dest)
-                      (loop (cdr locs))))))
-          (else (loop (cdr locs))))))
+          (else
+           (receive (dir version-from-path)
+               (locate-local-egg-dir (car locs) name version)
+             (if dir
+                 (let* ((eggfile (make-pathname dir name +egg-extension+))
+                        (info (validate-egg-info (load-egg-info eggfile)))
+                        (rversion
+                         ;; If version-from-path is non-#f, prefer it
+                         ;; over rversion, as it means the egg author
+                         ;; actually tagged the egg.  rversion might
+                         ;; be outdated in case the egg author forgot
+                         ;; to bump it in the .egg file.
+                         (or version-from-path
+                             (get-egg-property info 'version))))
+                   (d "trying location ~a ...~%" dir)
+                   (if (or (not rversion)
+                           (not version)
+                           (version>=? rversion version))
+                       (begin
+                         (copy-egg-sources dir dest)
+                         (write-cache-metadata name (or rversion version)))
+                       (loop (cdr locs))))
+                 (loop (cdr locs))))))))
+
 
 (define (copy-egg-sources from to)
   ;;XXX should probably be done manually, instead of calling tool
@@ -536,11 +589,15 @@
                     (or (and versions
                              (every (cut version>=? lversion <>) versions))
                         (loop (cdr srvs)))))))
-          ((probe-dir (make-pathname (car locs) name)) =>
-           (lambda (dir)
-             ;; for locally available eggs, check set of files and
-             ;; timestamps
-             (compare-trees dir cached)))
+          ;; The order of probe-dir's here is important.  First try
+          ;; the path with version, then the path without version.
+          ((or (probe-dir (make-pathname (list (car locs) name)
+                                         (->string lversion)))
+               (probe-dir (make-pathname (car locs) name)))
+           => (lambda (dir)
+                ;; for locally available eggs, check set of files and
+                ;; timestamps
+                (compare-trees dir cached)))
           (else (loop (cdr locs))))))
 
 (define (compare-trees there here)
@@ -550,8 +607,8 @@
           (hfs (directory here)))
       (every (lambda (f) 
                (and (member f hfs)
-                    (let ((tf2 (string-append there "/" f))
-                          (hf2 (string-append here "/" f)))
+                    (let ((tf2 (make-pathname there f))
+                          (hf2 (make-pathname here f)))
                       (and (<= (file-modification-time tf2)
                                (file-modification-time hf2))
                            (if (directory-exists? tf2)
@@ -813,9 +870,10 @@
     (lambda (egg)
       (let* ((name (car egg))
              (dir (cadr egg))
+             (metadata-dir (make-pathname cache-metadata-directory name))
              (eggfile (make-pathname dir name +egg-extension+))
              (info (load-egg-info eggfile))
-             (vfile (make-pathname dir +version-file+))
+             (vfile (make-pathname metadata-dir +version-file+))
              (ver (and (file-exists? vfile)
                        (with-input-from-file vfile read))))
         (when (or host-extension 
@@ -996,10 +1054,14 @@
           (for-each
             (lambda (egg)
               (let* ((name (if (pair? egg) (car egg) egg))
-                     (dname (make-pathname cache-directory name)))
-                (when (file-exists? dname)
-                  (d "purging ~a from cache at ~a~%" name dname)
-                  (delete-directory dname #t))))
+                     (cache-dir (make-pathname cache-directory name))
+                     (metadata-dir (make-pathname cache-metadata-directory name)))
+                (when (file-exists? cache-dir)
+                  (d "purging ~a from cache at ~a~%" name cache-dir)
+                  (delete-directory cache-dir #t))
+                (when (file-exists? metadata-dir)
+                  (d "purging metadata of ~a from cache at ~a~%" name metadata-dir)
+                  (delete-directory metadata-dir #t))))
             eggs))))
 
 
@@ -1040,6 +1102,9 @@ usage: chicken-install [OPTION ...] [NAME[:VERSION] ...]
        -force                   don't ask, install even if versions don't match
   -k   -keep                    keep temporary files
   -s   -sudo                    use external command to elevate privileges for filesystem operations
+  -l   -location DIRECTORY      get egg sources from DIRECTORY.  May be provided multiple times.
+                                Locations specified on the command line have precedence over the
+                                ones specified in setup.defaults.
   -r   -retrieve                only retrieve egg into cache directory, don't install (giving `-r'
                                 more than once implies `-recursive')
        -recursive               if `-retrieve' is given, retrieve also dependencies
@@ -1136,6 +1201,13 @@ EOF
                   ((member arg '("-s" "-sudo"))
                    (set! sudo-install #t)
                    (loop (cdr args)))
+                  ((member arg '("-l" "-location"))
+                   (when (null? (cdr args))
+                     (fprintf (current-error-port) "-l|-location: missing argument.~%")
+                     (exit 1))
+                   (set! default-locations
+                     (append (list (cadr args)) default-locations))
+                   (loop (cddr args)))
                   ((member arg '("-n" "-no-install"))
                    (set! no-install #t)
                    (loop (cdr args)))
