@@ -1,6 +1,6 @@
 ;;;; chicken-status.scm
 ;
-; Copyright (c) 2008-2021, The CHICKEN Team
+; Copyright (c) 2008-2022, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -112,7 +112,9 @@
     (let ((version
 	   (or (let ((info (read-info egg dir ext)))
 		 (and info (get-egg-property info 'version)))
-	       (let ((file (chicken.load#find-file +version-file+ dir)))
+               (let ((file (file-exists?
+                            (make-pathname (list cache-metadata-directory egg)
+                                           +version-file+))))
 		 (and file (with-input-from-file file read)))
 	       "unknown")))
       (print (format-string (string-append egg " ")
