@@ -1,6 +1,6 @@
 ;;;; csc.scm - Driver program for the CHICKEN compiler - felix -*- Scheme -*-
 ;
-; Copyright (c) 2008-2021, The CHICKEN Team
+; Copyright (c) 2008-2022, The CHICKEN Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -999,9 +999,7 @@ EOF
                                   TARGET_RUN_LIB_HOME)
                        lib))))
 	" " 
-	target) )
-      (when gui
-	(rez target)))
+	target) ))
     (unless keep-files 
       (for-each $delete-file
         (append generated-object-files
@@ -1056,23 +1054,6 @@ EOF
 
 ;;; Helper procedures:
 
-(define-constant +hairy-chars+ '(#\\ #\#))
-
-(define (cleanup s)
-  (let* ((q #f)
-	 (s (list->string
-	     (let fold ([s (string->list s)])
-	       (if (null? s) 
-		   '()
-		   (let ([c (car s)])
-		     (cond ((memq c +hairy-chars+) (cons* #\\ c (fold (cdr s))))
-			   (else
-			    (when (char-whitespace? c) (set! q #t))
-			    (cons c (fold (cdr s))) ) ) ) ) ) ) ) )
-    (if q 
-	(string-append "\"" (string-translate* s '(("\"" . "\\\""))) "\"")
-	s) ) )
-
 ;; Simpler replacement for SRFI-13's string-any
 (define (string-any criteria s)
   (let ((end (string-length s)))
@@ -1083,13 +1064,7 @@ EOF
             (or (criteria c)
                 (lp i1)))))))
 
-(define (quote-option x)
-  (cond ((string-any (cut char=? #\" <>) x) x)
-	((string-any (lambda (c)
-		       (or (char-whitespace? c) (memq c +hairy-chars+)) )
-		     x)
-	 (cleanup x))
-	(else x) ))
+(define quote-option qs)
 
 (define last-exit-code #f)
 
@@ -1118,13 +1093,6 @@ EOF
     (print "rm " str) )
   (unless dry-run (delete-file str) ))
 
-(define (rez file)
-  ;; see also: http://www.cocan.org/getting_started_with_ocaml_on_mac_os_x
-  (command 
-   (sprintf "/Developer/Tools/Rez -t APPL -o ~a ~a"
-     (quotewrap file)
-     (quotewrap (make-pathname home "mac.r")))))
-
 (define (create-win-manifest prg rcfname)
   (when verbose (print "generating " rcfname))
   (with-output-to-file rcfname
