@@ -1,6 +1,6 @@
 ;;;; csi.scm - Interpreter stub for CHICKEN
 ;
-; Copyright (c) 2008-2021, The CHICKEN Team
+; Copyright (c) 2008-2022, The CHICKEN Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -41,7 +41,7 @@ EOF
 ) )
 
 (module chicken.csi
-  (editor-command toplevel-command set-describer!)
+  (editor-command toplevel-command set-describer! default-evaluator)
 
 (import scheme
 	chicken.base
@@ -260,8 +260,6 @@ EOF
   (or (##core#inline "C_i_tty_forcedp")
       (##sys#tty-port? ##sys#standard-input)))
 
-(set! ##sys#break-on-error #f)
-
 (set! ##sys#read-prompt-hook
   (let ([old ##sys#read-prompt-hook])
     (lambda ()
@@ -279,10 +277,10 @@ EOF
 	 (set! command-table (cons (list name proc help) command-table))))
   (##sys#void))
 
-(define csi-eval
+(define default-evaluator
   (let ((eval eval)
 	(load-noisily load-noisily)
-	(read read)
+	(read (lambda () (chicken.syntax#read-with-source-info (current-input-port)))) ; OBSOLETE - after bootstrapping we can get rid of this explicit namespacing
 	(read-line read-line)
 	(display display)
 	(string-split string-split)
@@ -291,7 +289,7 @@ EOF
 	(pretty-print pretty-print)
 	(values values) )
     (lambda (form)
-      (cond ((eof-object? form) (exit))
+      (cond ((eof-object? form) (quit))
 	    ((and (pair? form)
 		  (eq? 'unquote (##sys#slot form 0)) )
 	     (let ((cmd (cadr form)))
@@ -594,6 +592,9 @@ EOF
 	    ((eq? x #t) (fprintf out "boolean true~%"))
 	    ((eq? x #f) (fprintf out "boolean false~%"))
 	    ((null? x) (fprintf out "empty list~%"))
+            ((##core#inline "C_bwpp" x) 
+             ;; TODO: replace with bwp-object? later
+             (fprintf out "broken weak pointer~%"))
 	    ((eof-object? x) (fprintf out "end-of-file object~%"))
 	    ((eq? (##sys#void) x) (fprintf out "unspecified object~%"))
 	    ((fixnum? x)
@@ -659,6 +660,9 @@ EOF
 	       (##sys#slot x 7)
 	       (##sys#slot x 3)
 	       (##sys#peek-unsigned-integer x 0) ) )
+            ((not (##core#inline "C_blockp" x)) 
+             ;; catch immediates here, as ##sys#locative? crashes on non-block
+             (fprintf out "unknown immediate object~%"))
 	    ((##sys#locative? x)
 	     (fprintf out "locative~%  pointer ~X~%  index ~A~%  type ~A~%"
 	       (##sys#peek-unsigned-integer x 0)
@@ -926,7 +930,7 @@ EOF
 		 (else (find (cdr ks))) ) ) ) ) )
 
 (define-constant short-options 
-  '(#\k #\s #\h #\D #\e #\i #\R #\b #\n #\q #\w #\- #\I #\p #\P) )
+  '(#\k #\s #\h #\D #\e #\i #\R #\b #\n #\q #\w #\- #\I #\p #\P #\K) )
 
 (define-constant long-options
   '("-ss" "-sx" "-script" "-version" "-help" "--help" "-feature" "-no-feature" "-eval"
@@ -973,6 +977,24 @@ EOF
 (define-constant complex-options
   '("-D" "-feature" "-I" "-include-path" "-K" "-keyword-style" "-no-feature") )
 
+(define (string-trim str)
+  (let loop ((front 0)
+	     (back (string-length str)))
+    (cond ((= front back) "")
+	  ((char-whitespace? (string-ref str front))
+	   (loop (add1 front) back))
+	  ((char-whitespace? (string-ref str (sub1 back)))
+	   (loop front (sub1 back)))
+	  (else (substring str front back)))))
+   
+(define (string->extension-name str)
+  (let ((str (string-trim str)))
+    (if (and (positive? (string-length str))
+             (char=? #\( (string-ref str 0)))
+        (handle-exceptions ex 
+          (##sys#error "invalid import specification" str)
+          (with-input-from-string str read))
+        (string->symbol str))))
 
 (define (run)
   (let* ([extraopts (parse-option-string (or (get-environment-variable "CSI_OPTIONS") ""))]
@@ -1000,13 +1022,10 @@ EOF
     (let* ([eval? (member* '("-e" "-p" "-P" "-eval" "-print" "-pretty-print") args)]
 	   [batch (or script (member* '("-b" "-batch") args) eval?)]
 	   [quietflag (member* '("-q" "-quiet") args)]
-	   [quiet (or script quietflag eval?)]
-	   [ipath (map chop-separator 
-		       (##sys#split-path
-			(or (get-environment-variable "CHICKEN_INCLUDE_PATH") "")))])
+	   [quiet (or script quietflag eval?)])
       (define (collect-options opt)
 	(let loop ([opts args])
-	  (cond [(member opt opts) 
+	  (cond [(member opt opts)
 		 => (lambda (p)
 		      (if (null? (cdr p))
 			  (##sys#error "missing argument to command-line option" opt)
@@ -1024,8 +1043,9 @@ EOF
                 ((and home-fn (file-exists? home-fn))
                  (load home-fn) ) ) ) )
       (define (evalstring str #!optional (rec (lambda _ (void))))
-	(let ((in (open-input-string str)))
-	  (do ([x (read in) (read in)])
+	(let ((in (open-input-string str))
+	      (read-with-source-info chicken.syntax#read-with-source-info)) ; OBSOLETE - after bootstrapping we can get rid of this explicit namespacing
+	  (do ([x (read-with-source-info in) (read-with-source-info in)])
 	      ((eof-object? x))
 	    (rec (receive (eval x))) ) ) )
       (when (member* '("-h" "-help" "--help") args)
@@ -1049,12 +1069,11 @@ EOF
       (for-each register-feature! (collect-options "-feature"))
       (for-each register-feature! (collect-options "-D"))
       (for-each unregister-feature! (collect-options "-no-feature"))
-      (set! ##sys#include-pathnames 
+      (set! ##sys#include-pathnames
 	(delete-duplicates
 	 (append (map chop-separator (collect-options "-include-path"))
 		 (map chop-separator (collect-options "-I"))
-		 ##sys#include-pathnames
-		 ipath)
+		 ##sys#include-pathnames)
 	 string=?) )
       (when kwstyle
 	(cond [(not (pair? (cdr kwstyle)))
@@ -1092,16 +1111,16 @@ EOF
 	(set! ##sys#notices-enabled #f))
       (do ([args args (cdr args)])
 	  ((null? args)
+	   (register-repl-history!)
 	   (unless batch
-	     (register-repl-history!)
-	     (repl csi-eval)
+	     (repl default-evaluator)
 	     (##sys#write-char-0 #\newline ##sys#standard-output) ) )
 	(let* ((arg (car args)))
 	  (cond ((member arg simple-options))
 		((member arg complex-options)
 		 (set! args (cdr args)) )
 		((or (string=? "-R" arg) (string=? "-require-extension" arg))
-		 (eval `(import ,(string->symbol (cadr args))))
+		 (eval `(import ,(string->extension-name (cadr args))))
 		 (set! args (cdr args)) )
 		((or (string=? "-e" arg) (string=? "-eval" arg))
 		 (evalstring (cadr args))
