@@ -1,6 +1,6 @@
 ;;;; eval.scm - Interpreter for CHICKEN
 ;
-; Copyright (c) 2008-2021, The CHICKEN Team
+; Copyright (c) 2008-2022, The CHICKEN Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -109,10 +109,11 @@
 		((eq? x (##sys#slot lst 0)) i)
 		(else (loop (##sys#slot lst 1) (fx+ i 1))) ) ) )
 
-      (define (emit-trace-info tf info cntr e v) 
+      (define (emit-trace-info tf ln info cntr e v)
 	(when tf
 	  (##core#inline 
-	   "C_emit_eval_trace_info" 
+	   "C_emit_trace_info"
+	   ln
 	   info
 	   (##sys#make-structure 'frameinfo cntr e v)
 	   (thread-id ##sys#current-thread) ) ) )
@@ -120,7 +121,8 @@
       (define (emit-syntax-trace-info tf info cntr) 
 	(when tf
 	  (##core#inline
-	   "C_emit_syntax_trace_info"
+	   "C_emit_trace_info"
+	   (or (get-line-number info) "<syntax>")
 	   info
 	   cntr
            (thread-id ##sys#current-thread) ) ) )
@@ -128,6 +130,13 @@
       (define (decorate p ll h cntr)
 	(eval-decorator p ll h cntr))
 
+      (define (handle-expansion-result outer-ln)
+	(lambda (input output)
+	  (and-let* (((not (eq? input output)))
+		     (ln (or (get-line-number input) outer-ln)))
+	    (##sys#update-line-number-database! output ln))
+	  output))
+
       (define (compile x e h tf cntr tl?)
 	(cond ((keyword? x) (lambda v x))
 	      ((symbol? x)
@@ -136,8 +145,9 @@
 			(let ((var (cond ((not (symbol? j)) x) ; syntax?
 					 ((assq x (##sys#current-environment)) j)
 					 ((not static)
-					  (##sys#alias-global-hook j #f cntr))
-					 (else #f))))
+                                          (##sys#alias-global-hook j #f cntr))
+                                         ((not (eq? x j)) j) ; has macro-alias
+                                         (else #f))))
 			  (when (and ##sys#unbound-in-eval
 				     (or (not var)
 					 (not (##sys#symbol-has-toplevel-binding? var))))
@@ -182,6 +192,8 @@
 		   (lambda v #f) ) ]
 	      ((or (char? x)
 		   (eof-object? x)
+		   (##core#inline "C_bwpp" x) ; TODO: Remove once we have a bootstrapping libchicken with bwp-object?
+		   ;;(bwp-object? x)
 		   (string? x)
 		   (blob? x)
 		   (vector? x)
@@ -191,7 +203,10 @@
 	       (##sys#syntax-error/context "illegal non-atomic object" x)]
 	      [(symbol? (##sys#slot x 0))
 	       (emit-syntax-trace-info tf x cntr)
-	       (let ((x2 (expand x (##sys#current-environment))))
+	       (let* ((ln (get-line-number x))
+		      (x2 (fluid-let ((chicken.syntax#expansion-result-hook
+				       (handle-expansion-result ln)))
+			    (expand x (##sys#current-environment)))))
 		 (d `(EVAL/EXPANDED: ,x2))
 		 (if (not (eq? x2 x))
 		     (compile x2 e h tf cntr tl?)
@@ -250,6 +265,7 @@
 			 ((##core#ensure-toplevel-definition)
 			  (unless tl?
 			    (##sys#error "toplevel definition in non-toplevel context for variable" (cadr x)))
+                          (##sys#put/restore! (cadr x) '##sys#override 'value)
 			  (compile
 			   '(##core#undefined) e #f tf cntr #f))
 
@@ -493,6 +509,7 @@
 				 (name (rename var)))
 			    (when (and static (not (assq var (##sys#current-environment))))
 			      (##sys#error 'eval "environment is not mutable" evalenv var))
+                            (##sys#put/restore! name '##sys#override 'syntax)
 			    (##sys#register-syntax-export 
 			     name (##sys#current-module)
 			     body)	; not really necessary, it only shouldn't be #f
@@ -506,7 +523,7 @@
 			  (compile '(##core#undefined) e #f tf cntr #f))
 
 			 ((##core#let-compiler-syntax)
-			  (compile 
+			  (compile
 			   (##sys#canonicalize-body (cddr x) (##sys#current-environment) #f)
 			   e #f tf cntr #f))
 
@@ -514,14 +531,14 @@
 			  (##sys#include-forms-from-file
 			   (cadr x)
 			   (caddr x)
-			   (lambda (forms)
-			     (compile
-			      (if (pair? (cdddr x)) ; body?
-				  (##sys#canonicalize-body
-				   (append forms (cadddr x))
-				   (##sys#current-environment))
-				  `(##core#begin ,@forms))
-			      e #f tf cntr tl?))))
+			   (lambda (forms path)
+			     (let ((code (if (pair? (cdddr x)) ; body?
+			 	     	     (##sys#canonicalize-body
+				               (append forms (cadddr x))
+				               (##sys#current-environment))
+				             `(##core#begin ,@forms))))
+			       (fluid-let ((##sys#current-source-filename path))
+			         (compile code e #f tf cntr tl?))))))
 
 			 ((##core#let-module-alias)
 			  (##sys#with-module-aliases
@@ -596,9 +613,8 @@
 
 			 [(##core#require)
 			  (let ((lib (cadr x))
-			  	 (mod (and (pair? (cddr x)) (caddr x))))
-                            (let-values (((reqform _) (##sys#process-require lib mod #f)))
-			      (compile reqform e #f tf cntr #f)))]
+				(mod (and (pair? (cddr x)) (caddr x))))
+			    (compile (##sys#process-require lib mod #f) e #f tf cntr #f))]
 
 			 [(##core#elaborationtimeonly ##core#elaborationtimetoo) ; <- Note this!
 			  (##sys#eval/meta (cadr x))
@@ -676,37 +692,38 @@
 		       (compile (##sys#slot x 0) e #f tf cntr #f)))
 	       (args (##sys#slot x 1))
 	       (argc (checked-length args))
-	       (info x) )
+	       (info x)
+	       (ln (or (get-line-number info) "<eval>")))
 	  (case argc
 	    ((#f) (##sys#syntax-error/context "malformed expression" x))
 	    ((0) (lambda (v)
-		   (emit-trace-info tf info cntr e v)
+		   (emit-trace-info tf ln info cntr e v)
 		   ((##core#app fn v))))
 	    ((1) (let ((a1 (compile (##sys#slot args 0) e #f tf cntr #f)))
 		   (lambda (v)
-		     (emit-trace-info tf info cntr e v)
+		     (emit-trace-info tf ln info cntr e v)
 		     ((##core#app fn v) (##core#app a1 v))) ) )
 	    ((2) (let* ((a1 (compile (##sys#slot args 0) e #f tf cntr #f))
 			(a2 (compile (##core#inline "C_u_i_list_ref" args 1) e #f tf cntr #f)) )
 		   (lambda (v)
-		     (emit-trace-info tf info cntr e v)
+		     (emit-trace-info tf ln info cntr e v)
 		     ((##core#app fn v) (##core#app a1 v) (##core#app a2 v))) ) )
 	    ((3) (let* ((a1 (compile (##sys#slot args 0) e #f tf cntr #f))
 			(a2 (compile (##core#inline "C_u_i_list_ref" args 1) e #f tf cntr #f))
 			(a3 (compile (##core#inline "C_u_i_list_ref" args 2) e #f tf cntr #f)) )
 		   (lambda (v)
-		     (emit-trace-info tf info cntr e v)
+		     (emit-trace-info tf ln info cntr e v)
 		     ((##core#app fn v) (##core#app a1 v) (##core#app a2 v) (##core#app a3 v))) ) )
 	    ((4) (let* ((a1 (compile (##sys#slot args 0) e #f tf cntr #f))
 			(a2 (compile (##core#inline "C_u_i_list_ref" args 1) e #f tf cntr #f))
 			(a3 (compile (##core#inline "C_u_i_list_ref" args 2) e #f tf cntr #f))
 			(a4 (compile (##core#inline "C_u_i_list_ref" args 3) e #f tf cntr #f)) )
 		   (lambda (v)
-		     (emit-trace-info tf info cntr e v)
+		     (emit-trace-info tf ln info cntr e v)
 		     ((##core#app fn v) (##core#app a1 v) (##core#app a2 v) (##core#app a3 v) (##core#app a4 v))) ) )
 	    (else (let ((as (##sys#map (lambda (a) (compile a e #f tf cntr #f)) args)))
 		    (lambda (v)
-		      (emit-trace-info tf info cntr e v)
+		      (emit-trace-info tf ln info cntr e v)
 		      (apply (##core#app fn v) (##sys#map (lambda (a) (##core#app a v)) as))) ) ) ) ) )
 
       (compile exp env #f (fx> (##sys#eval-debug-level) 0) cntr tl?) ) ) )
@@ -837,6 +854,7 @@
 		   define-interface
 		   delay-force
 		   export
+                   export/rename
 		   functor
 		   import
 		   import-for-syntax
@@ -892,6 +910,7 @@
 	chicken.foreign
 	chicken.internal
 	chicken.platform
+	chicken.syntax
 	chicken.time)
 
 (include "mini-srfi-1.scm")
@@ -904,6 +923,7 @@
 
 ;;; Core unit information
 
+;; this maps built-in library names to require forms when the mapping isn't 1:1
 (define-constant core-unit-requirements
   '((chicken.foreign
      . (##core#require-for-syntax chicken-ffi-syntax))
@@ -912,6 +932,9 @@
 	(##core#require-for-syntax chicken-syntax)
 	(##core#require library)))))
 
+;; this list contains built-in units that are provided by libchicken
+;; and should not be treated as separate extension libraries during
+;; linking (they are omitted from types/inline/link files etc.)
 (define-constant core-units
   '(chicken-syntax chicken-ffi-syntax continuation data-structures
     debugger-client eval eval-modules expand extras file internal
@@ -934,17 +957,10 @@
 
 (define ##sys#load-dynamic-extension default-load-library-extension)
 
-(define (chicken.load#core-unit? id) ; used by batch-driver.scm
+(define (chicken.load#core-library? id) ; used by core.scm
   (or (memq id core-units)
       (assq id core-unit-requirements)))
 
-; these are actually in unit extras, but that is used by default
-
-(define-constant builtin-features
-  '(srfi-30 srfi-46 srfi-61 srfi-62                     ; runtime
-    srfi-0 srfi-2 srfi-8 srfi-9 srfi-11 srfi-15 srfi-16 ; syntax
-    srfi-17 srfi-26 srfi-31 srfi-55 srfi-87 srfi-88))   ; syntax cont
-
 (define default-dynamic-load-libraries
   (case (software-version)
     ((cygwin) cygwin-default-dynamic-load-libraries)
@@ -1014,8 +1030,7 @@
   (##sys#make-c-string (##sys#string-append "C_" (toplevel name)) loc))
 
 (define load/internal
-  (let ((read read)
-	(write write)
+  (let ((write write)
 	(display display)
 	(newline newline)
 	(eval eval)
@@ -1067,7 +1082,8 @@
 	     (fluid-let ((##sys#read-error-with-line-number #t)
 			 (##sys#current-load-filename fname)
 			 (##sys#current-source-filename fname))
-	       (let ((in (if fname (open-input-file fname) input)))
+	       (let ((in (if fname (open-input-file fname) input))
+		     (read-with-source-info chicken.syntax#read-with-source-info)) ; OBSOLETE - after bootstrapping we can get rid of this explicit namespacing
 		 (##sys#dynamic-wind
 		  (lambda () #f)
 		  (lambda ()
@@ -1079,8 +1095,8 @@
 			  "unable to load compiled module - "
 			  (or _dlerror "unknown reason"))
 			 fname)))
-		    (let ((x1 (read in)))
-		      (do ((x x1 (read in)))
+		    (let ((x1 (read-with-source-info in)))
+		      (do ((x x1 (read-with-source-info in)))
 			  ((eof-object? x))
 			(when printer (printer x))
 			(##sys#call-with-values
@@ -1153,22 +1169,23 @@
   (load-unit unit-name lib 'load-library))
 
 (define ##sys#include-forms-from-file
-  (let ((with-input-from-file with-input-from-file)
-	(read read)
-	(reverse reverse))
+  (let ((call-with-input-file call-with-input-file)
+        (reverse reverse))
     (lambda (filename source k)
-      (let ((path (##sys#resolve-include-filename filename #t #f source)))
-	(when (not path)
-	  (##sys#signal-hook #:file-error 'include "cannot open file" filename))
-	(when (load-verbose)
-	  (print "; including " path " ..."))
-	(with-input-from-file path
-	  (lambda ()
-	    (fluid-let ((##sys#current-source-filename path))
-	      (do ((x (read) (read))
-		   (xs '() (cons x xs)))
-		  ((eof-object? x)
-		   (k (reverse xs)))))))))))
+      (let ((path (##sys#resolve-include-filename filename #t #f source))
+            (read-with-source-info chicken.syntax#read-with-source-info)) ; OBSOLETE - after bootstrapping we can get rid of this explicit namespacing
+        (when (not path)
+          (##sys#signal-hook #:file-error 'include "cannot open file" filename))
+        (when (load-verbose)
+          (print "; including " path " ..."))
+        (call-with-input-file path
+          (lambda (in)
+            (k (fluid-let ((##sys#current-source-filename path))
+                 (do ((x (read-with-source-info in) (read-with-source-info in))
+                      (xs '() (cons x xs)))
+                     ((eof-object? x)
+                      (reverse xs))))
+               path)))))))
 
 
 ;;; Extensions:
@@ -1251,31 +1268,22 @@
 (define (##sys#process-require lib mod compile-mode)
   (let ((mod (or (eq? lib mod) mod)))
     (cond
-      ((assq lib core-unit-requirements) => 
-        (lambda (a) (values (cdr a) #t)))
-      ((memq lib builtin-features) 
-        (values '(##core#undefined) #t))
+      ((assq lib core-unit-requirements) => cdr)
       ((memq lib core-units)
-        (values
-          (if compile-mode
-   	      `(##core#callunit ,lib)
-	      `(chicken.load#load-unit (##core#quote ,lib)
-	  			       (##core#quote #f)
-				       (##core#quote #f)))
-          #t))
+       (if compile-mode
+           `(##core#callunit ,lib)
+           `(chicken.load#load-unit (##core#quote ,lib)
+                                    (##core#quote #f)
+                                    (##core#quote #f))))
       ((eq? compile-mode 'static)
-       (values `(##core#callunit ,lib) #f))
+       `(##core#callunit ,lib))
       (else
-       (values
-          `(chicken.load#load-extension (##core#quote ,lib)
-				        (##core#quote ,mod)
-				        (##core#quote #f))
-          #f)))))
+       `(chicken.load#load-extension (##core#quote ,lib)
+                                     (##core#quote ,mod)
+                                     (##core#quote #f))))))
 
 ;;; Find included file:
 
-(define ##sys#include-pathnames (list (chicken-home)))
-
 (define ##sys#resolve-include-filename
   (let ((string-append string-append) )
     (lambda (fname exts repo source)
