@@ -1,6 +1,6 @@
 ;;;; expand.scm - The HI/LO expander
 ;
-; Copyright (c) 2008-2021, The CHICKEN Team
+; Copyright (c) 2008-2022, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -37,6 +37,7 @@
 (module chicken.syntax
   (expand
    get-line-number
+   read-with-source-info
    strip-syntax
    syntax-error
    er-macro-transformer
@@ -48,12 +49,14 @@
 	chicken.fixnum
 	chicken.internal
 	chicken.keyword
-	chicken.platform)
+	chicken.platform
+	chicken.string)
 
 (include "common-declarations.scm")
 (include "mini-srfi-1.scm")
 
 (define-syntax d (syntax-rules () ((_ . _) (void))))
+;(define-syntax d (syntax-rules () ((_ args ...) (print args ...))))
 
 ;; Macro to avoid "unused variable map-se" when "d" is disabled
 (define-syntax map-se
@@ -63,10 +66,6 @@
 	    (cons (car a) (if (symbol? (cdr a)) (cdr a) '<macro>)))
 	  ?se))))
 
-(set! ##sys#features
-  (append '(#:srfi-0 #:srfi-2 #:srfi-6 #:srfi-9 #:srfi-46 #:srfi-55 #:srfi-61)
-	  ##sys#features))
-
 (define-alias dd d)
 (define-alias dm d)
 (define-alias dx d)
@@ -263,10 +262,13 @@
 	    (let ((head2 (or (lookup head dse) head)))
 	      (unless (pair? head2)
 		(set! head2 (or (lookup head2 (##sys#macro-environment)) head2)) )
-	      (cond [(eq? head2 '##core#let)
+	      (cond ((and (pair? head2)
+                          (eq? (##sys#get head '##sys#override) 'value))
+                     (values exp #f))
+                    ((eq? head2 '##core#let)
 		     (##sys#check-syntax 'let body '#(_ 2) #f dse)
-		     (let ([bindings (car body)])
-		       (cond [(symbol? bindings) ; expand named let
+		     (let ((bindings (car body)))
+		       (cond ((symbol? bindings) ; expand named let
 			      (##sys#check-syntax 'let body '(_ #((variable _) 0) . #(_ 1)) #f dse)
 			      (let ([bs (cadr body)])
 				(values
@@ -277,8 +279,8 @@
 				       ,(map (lambda (b) (car b)) bs) ,@(cddr body))])
 				    ,bindings)
 				   ,@(##sys#map cadr bs) )
-				 #t) ) ]
-			     [else (values exp #f)] ) ) ]
+				 #t) ) )
+			     (else (values exp #f)) ) ) )
 		    ((and cs? (symbol? head2) (getp head2 '##compiler#compiler-syntax)) =>
 		     (lambda (cs)
 		       (let ((result (call-handler head (car cs) exp (cdr cs) #t)))
@@ -287,7 +289,7 @@
 				(when ##sys#compiler-syntax-hook
 				  (##sys#compiler-syntax-hook head2 result))
 				(loop result))))))
-		    [else (expand head exp head2)] ) )
+		    (else (expand head exp head2)) ) )
 	    (values exp #f) ) )
       (values exp #f) ) ) )
 
@@ -462,14 +464,26 @@
 (define ##sys#canonicalize-body
   (lambda (body #!optional (se (##sys#current-environment)) cs?)
     (define (comp s id)
-      (let ((f (lookup id se)))
-	(or (eq? s f)
-	    (case s
-	      ((define) (if f (eq? f define-definition) (eq? s id)))
-	      ((define-syntax) (if f (eq? f define-syntax-definition) (eq? s id)))
-	      ((define-values) (if f (eq? f define-values-definition) (eq? s id)))
-	      ((import) (if f (eq? f import-definition) (eq? s id)))
-	      (else (eq? s id))))))
+      (let ((f (or (lookup id se)
+                   (lookup id (##sys#macro-environment)))))
+        (and (or (not (symbol? f))
+                 (not (eq? (##sys#get id '##sys#override) 'value)))
+             (or (eq? f s) (eq? s id)))))
+    (define (comp-def def)
+      (lambda (id)
+        (let repeat ((id id))
+          (let ((f (or (lookup id se)
+                       (lookup id (##sys#macro-environment)))))
+            (and (or (not (symbol? f))
+                     (not (eq? (##sys#get id '##sys#override) 'value)))
+                 (or (eq? f def)
+                     (and (symbol? f) 
+                          (not (eq? f id))
+                          (repeat f))))))))
+    (define comp-define (comp-def define-definition))
+    (define comp-define-syntax (comp-def define-syntax-definition))
+    (define comp-define-values (comp-def define-values-definition))
+    (define comp-import (comp-def import-definition))
     (define (fini vars vals mvars body)
       (if (and (null? vars) (null? mvars))
 	  ;; Macro-expand body, and restart when defines are found.
@@ -484,13 +498,13 @@
 		    (if (and (pair? x)
 			     (let ((d (car x)))
 			       (and (symbol? d)
-				    (or (comp 'define d)
-					(comp 'define-values d)
-					(comp 'define-syntax d)
-					(comp '##core#begin d)
-					(comp 'import d)))))
+				    (or (comp '##core#begin d)
+                                        (comp-define d)
+					(comp-define-values d)
+					(comp-define-syntax d)
+					(comp-import d)))))
 			;; Stupid hack to avoid expanding imports
-			(if (comp 'import (car x))
+			(if (comp-import (car x))
 			    (loop rest (cons x exps))
 			    (cons
 			     '##core#begin
@@ -549,7 +563,7 @@
 	       ((and (list? (car body))
 		     (>= 3 (length (car body))) 
 		     (symbol? (caar body))
-		     (comp 'define-syntax (caar body)))
+		     (comp-define-syntax (caar body)))
 		(let ((def (car body)))
 		  ;; This check is insufficient, if introduced by
 		  ;; different expansions, but better than nothing:
@@ -563,6 +577,7 @@
       ;; Each #t in "mvars" indicates an MV-capable "var".  Non-MV
       ;; vars (#f in mvars) are 1-element lambda-lists for simplicity.
       (let loop ((body body) (vars '()) (vals '()) (mvars '()))
+        (d "BODY: " body)
 	(if (not (pair? body))
 	    (fini vars vals mvars body)
 	    (let* ((x (car body))
@@ -572,7 +587,7 @@
 	      (if (not (symbol? head))
 		  (fini vars vals mvars body)
 		  (cond
-		   ((comp 'define head)
+		   ((comp-define head)
 		     (##sys#check-syntax 'define x '(_ _ . #(_ 0)) #f se)
 		     (let loop2 ((x x))
 		       (let ((head (cadr x)))
@@ -599,10 +614,10 @@
 				      (cons (list (car head)) vars)
 				      (cons `(##core#lambda ,(cdr head) ,@(cddr x)) vals)
 				      (cons #f mvars)))))))
-		    ((comp 'define-syntax head)
+		    ((comp-define-syntax head)
 		     (##sys#check-syntax 'define-syntax x '(_ _ . #(_ 1)) se)
 		     (fini/syntax vars vals mvars body))
-		    ((comp 'define-values head)
+		    ((comp-define-values head)
 		     ;;XXX check for any of the variables being `define-values'
 		     (##sys#check-syntax 'define-values x '(_ lambda-list _) #f se)
 		     (loop rest (cons (cadr x) vars) (cons (caddr x) vals) (cons #t mvars)))
@@ -655,9 +670,12 @@
       (list 'define name exp) ) ) )
 
 
-;;; General syntax checking routine:
+;;; Line-number database management:
 
 (define ##sys#line-number-database #f)
+
+;;; General syntax checking routine:
+
 (define ##sys#syntax-error-culprit #f)
 (define ##sys#syntax-context '())
 
@@ -716,6 +734,44 @@
 			   (else (loop (cdr cx))))))))
 	  (##sys#syntax-error-hook (get-output-string out))))))
 
+;;; Hook for source information
+
+(define (alist-weak-cons k v lst)
+  (cons (##core#inline_allocate ("C_a_i_weak_cons" 3) k v) lst))
+
+(define (assq/drop-bwp! x lst)
+  (let lp ((lst lst)
+	   (prev #f))
+    (cond ((null? lst) #f)
+	  ((eq? x (caar lst)) (car lst))
+	  ((and prev
+		(##core#inline "C_bwpp" (caar lst)))
+	   (set-cdr! prev (cdr lst))
+	   (lp (cdr lst) prev))
+	  (else (lp (cdr lst) lst)))))
+
+(define (read-with-source-info-hook class data val)
+  (when (and (eq? 'list-info class) (symbol? (car data)))
+    (let ((old-value (or (hash-table-ref ##sys#line-number-database (car data)) '())))
+      (assq/drop-bwp! (car data) old-value) ;; Hack to clean out garbage values
+      (hash-table-set!
+       ##sys#line-number-database
+       (car data)
+       (alist-weak-cons
+	data (conc (or ##sys#current-source-filename "<stdin>") ":" val)
+	old-value ) )) )
+  data)
+
+(define-constant line-number-database-size 997) ; Copied from core.scm
+
+(define (read-with-source-info #!optional (in ##sys#standard-input))
+  ;; Initialize line number db on first use
+  (unless ##sys#line-number-database
+    (set! ##sys#line-number-database (make-vector line-number-database-size '())))
+  (##sys#check-input-port in #t 'read-with-source-info)
+  (##sys#read in read-with-source-info-hook) )
+
+
 (define (get-line-number sexp)
   (and ##sys#line-number-database
        (pair? sexp)
@@ -723,10 +779,49 @@
 	 (and (symbol? head)
 	      (cond ((hash-table-ref ##sys#line-number-database head)
 		     => (lambda (pl)
-			  (let ((a (assq sexp pl)))
+			  (let ((a (assq/drop-bwp! sexp pl)))
 			    (and a (cdr a)))))
 		    (else #f))))))
 
+;; TODO: Needs a better name - it extracts the name(?) and the source expression
+(define (##sys#get-line-2 exp)
+  (let* ((name (car exp))
+	 (lst (hash-table-ref ##sys#line-number-database name)))
+    (cond ((and lst (assq/drop-bwp! exp (cdr lst)))
+	   => (lambda (a) (values (car lst) (cdr a))) )
+	  (else (values name #f)) ) ) )
+
+(define (##sys#display-line-number-database)
+  (hash-table-for-each
+   (lambda (key val)
+     (when val
+       (let ((port (current-output-port)))
+	 (##sys#print key #t port)
+	 (##sys#print " " #f port)
+	 (##sys#print (map cdr val) #t port)
+	 (##sys#print "\n" #f port))) )
+   ##sys#line-number-database) )
+
+;;; Traverse expression and update line-number db with all contained calls:
+
+(define (##sys#update-line-number-database! exp ln)
+  (define (mapupdate xs)
+    (let loop ((xs xs))
+      (when (pair? xs)
+	(walk (car xs))
+	(loop (cdr xs)) ) ) )
+  (define (walk x)
+    (cond ((not (pair? x)))
+	  ((symbol? (car x))
+	   (let* ((name (car x))
+		  (old (or (hash-table-ref ##sys#line-number-database name) '())))
+	     (unless (assq x old)
+	       (hash-table-set! ##sys#line-number-database name (alist-cons x ln old)))
+	     (mapupdate (cdr x)) ) )
+	  (else (mapupdate x)) ) )
+  (walk exp) )
+
+
 (define-constant +default-argument-count-limit+ 99999)
 
 (define ##sys#check-syntax
@@ -826,7 +921,7 @@
 		    (cur (or (hash-table-ref ##sys#line-number-database name) '())) )
 	   (unless (assq new cur)
 	     (hash-table-set! ##sys#line-number-database name
-			      (alist-cons new ln cur))))
+			      (alist-weak-cons new ln cur))))
 	 new)
        (assert (list? se) "not a list" se) ;XXX remove later
        (define (rename sym)
@@ -1120,6 +1215,25 @@
 	(##sys#add-to-export-list mod exps))
       '(##core#undefined)))))
 
+(##sys#extend-macro-environment
+ 'export/rename '()
+ (##sys#er-transformer
+  (lambda (x r c)
+    (let ((exps (map (lambda (ren)
+                       (if (and (pair? ren) 
+                                (symbol? (car ren))
+                                (pair? (cdr ren))
+                                (symbol? (cadr ren))
+                                (null? (cddr ren)))
+                           (cons (car ren) (cadr ren))
+                           (##sys#syntax-error-hook "invalid item in export rename list" 
+                                                    ren)))
+                  (strip-syntax (cdr x))))
+          (mod (##sys#current-module)))
+      (when mod
+	(##sys#add-to-export/rename-list mod exps))
+      '(##core#undefined)))))
+
 (##sys#extend-macro-environment
  'reexport '()
  (##sys#er-transformer
@@ -1386,6 +1500,9 @@
                          (char? (car clause))
                          (string? (car clause))
                          (eof-object? (car clause))
+			 ;; TODO: Remove once we have a bootstrapping libchicken with bwp-object?
+			 (##core#inline "C_bwpp" (car clause))
+                         #;(bwp-object? (car clause))
                          (blob? (car clause))
                          (vector? (car clause))
                          (##sys#srfi-4-vector? (car clause))
@@ -1585,3 +1702,7 @@
   (fixup-macro-environment (##sys#macro-environment)))
 
 (define ##sys#meta-macro-environment (make-parameter (##sys#macro-environment)))
+
+;; register features
+
+(register-feature! 'srfi-0 'srfi-46 'srfi-61 'srfi-87)
