@@ -1,6 +1,6 @@
 ;;;; lfa2.scm - a lightweight "secondary" flow analysis
 ;
-; Copyright (c) 2012-2021, The CHICKEN Team
+; Copyright (c) 2012-2022, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -167,12 +167,16 @@
     ("C_a_i_bignum2" bignum)
     ("C_a_i_flonum_abs" float)
     ("C_a_i_flonum_acos" float)
+    ("C_a_i_flonum_acosh" float)
     ("C_a_i_flonum_actual_quotient_checked" float)
     ("C_a_i_flonum_asin" float)
+    ("C_a_i_flonum_asinh" float)
     ("C_a_i_flonum_atan2" float)
     ("C_a_i_flonum_atan" float)
+    ("C_a_i_flonum_atanh" float)
     ("C_a_i_flonum_ceiling" float)
     ("C_a_i_flonum_cos" float)
+    ("C_a_i_flonum_cosh" float)
     ("C_a_i_flonum_difference" float)
     ("C_a_i_flonum_exp" float)
     ("C_a_i_flonum_expt" float)
@@ -188,9 +192,12 @@
     ("C_a_i_flonum_round" float)
     ("C_a_i_flonum_round_proper" float)
     ("C_a_i_flonum_sin" float)
+    ("C_a_i_flonum_sinh" float)
     ("C_a_i_flonum_sqrt" float)
     ("C_a_i_flonum_tan" float)
+    ("C_a_i_flonum_tanh" float)
     ("C_a_i_flonum_times" float)
+    ("C_a_i_flonum_multiply_add" float)
     ("C_a_i_flonum_truncate" float)
     ("C_a_u_i_f64vector_ref" float)
     ("C_a_u_i_f32vector_ref" float)
@@ -201,6 +208,7 @@
   '(("C_a_i_flonum_plus" "C_ub_i_flonum_plus" op)
     ("C_a_i_flonum_difference" "C_ub_i_flonum_difference" op)
     ("C_a_i_flonum_times" "C_ub_i_flonum_times" op)
+    ("C_a_i_flonum_multiply_add" "C_ub_i_flonum_multiply_add" op)
     ("C_a_i_flonum_quotient" "C_ub_i_flonum_quotient" op)
     ("C_flonum_equalp" "C_ub_i_flonum_equalp" pred)
     ("C_flonum_greaterp" "C_ub_i_flonum_greaterp" pred)
@@ -217,6 +225,12 @@
     ("C_a_i_flonum_acos" "C_acos" op)
     ("C_a_i_flonum_atan" "C_atan" op)
     ("C_a_i_flonum_atan2" "C_atan2" op)
+    ("C_a_i_flonum_sinh" "C_sinh" op)
+    ("C_a_i_flonum_cosh" "C_cosh" op)
+    ("C_a_i_flonum_tanh" "C_tanh" op)
+    ("C_a_i_flonum_asinh" "C_asinh" op)
+    ("C_a_i_flonum_acosh" "C_acosh" op)
+    ("C_a_i_flonum_atanh" "C_atanh" op)
     ("C_a_i_flonum_exp" "C_exp" op)
     ("C_a_i_flonum_expr" "C_pow" op)
     ("C_a_i_flonum_log" "C_log" op)
@@ -253,6 +267,8 @@
 	    ((list? lit) 'list)
 	    ((pair? lit) 'pair)
 	    ((eof-object? lit) 'eof)
+	    ;; TODO: Remove once we have a bootstrapping libchicken with bwp-object?
+	    ((##core#inline "C_bwpp" lit) #;(bwp-object? lit) 'bwp)
 	    ((vector? lit) 'vector)
 	    ((and (not (##sys#immediate? lit)) (##sys#generic-structure? lit))
 	     `(struct ,(##sys#slot lit 0)))
