@@ -1,6 +1,6 @@
 ;;;; lolevel.scm - Low-level routines for CHICKEN
 ;
-; Copyright (c) 2008-2021, The CHICKEN Team
+; Copyright (c) 2008-2022, The CHICKEN Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -554,7 +554,7 @@ EOF
 
 (module chicken.locative
   (locative? make-locative make-weak-locative
-   locative-ref locative-set! locative->object)
+   locative-ref locative-set! locative->object locative-index)
 
 (import scheme chicken.base)
 
@@ -597,5 +597,8 @@ EOF
 (define (locative->object x)
   (##core#inline "C_i_locative_to_object" x))
 
+(define (locative-index x)
+  (##core#inline "C_i_locative_index" x))
+
 (define (locative? x)
   (and (##core#inline "C_blockp" x) (##core#inline "C_locativep" x))))
