@@ -1,6 +1,6 @@
 ;;;; modules.scm - module-system support
 ;
-; Copyright (c) 2011-2021, The CHICKEN Team
+; Copyright (c) 2011-2022, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -90,12 +90,13 @@
 	module-meta-expressions set-module-meta-expressions!
 	module-defined-syntax-list set-module-defined-syntax-list!
 	module-saved-environments set-module-saved-environments!
-	module-iexports set-module-iexports!))
+	module-iexports set-module-iexports!
+        module-rename-list set-module-rename-list!))
 
 (define-record-type module
   (%make-module name library export-list defined-list exist-list defined-syntax-list
 		undefined-list import-forms meta-import-forms meta-expressions 
-		vexports sexports iexports saved-environments) 
+		vexports sexports iexports saved-environments rename-list) 
   module?
   (name module-name)			; SYMBOL
   (library module-library)		; SYMBOL
@@ -111,7 +112,8 @@
   (sexports module-sexports set-module-sexports!)	      ; ((SYMBOL SE TRANSFORMER) ...)
   (iexports module-iexports set-module-iexports!)	      ; ((SYMBOL . SYMBOL) ...)
   ;; for csi's ",m" command, holds (<env> . <macroenv>)
-  (saved-environments module-saved-environments set-module-saved-environments!))
+  (saved-environments module-saved-environments set-module-saved-environments!)
+  (rename-list module-rename-list set-module-rename-list!))
 
 (define ##sys#module-name module-name)
 
@@ -121,8 +123,9 @@
    (module-vexports m)
    (module-sexports m)))
 
-(define (make-module name lib explist vexports sexports iexports)
-  (%make-module name lib explist '() '() '() '() '() '() '() vexports sexports iexports #f))
+(define (make-module name lib explist vexports sexports iexports #!optional (renames '()))
+  (%make-module name lib explist '() '() '() '() '() '() '() vexports sexports iexports #f
+                renames))
 
 (define (##sys#register-module-alias alias name)
   (##sys#module-alias-environment
@@ -181,6 +184,11 @@
 	  (set-module-exist-list! mod (append el exps)))
 	(set-module-export-list! mod (append xl exps)))))
 
+(define (##sys#add-to-export/rename-list mod renames)
+  (let ((rl (module-rename-list mod)))
+    (set-module-rename-list! mod (append rl renames))
+    (##sys#add-to-export-list mod (map car renames))))
+
 (define (##sys#toplevel-definition-hook sym renamed exported?) #f)
 
 (define (##sys#register-meta-expression exp)
@@ -303,8 +311,7 @@
 	    ((or (eq? last-se (car ses)) (null? (car ses)))
 	     (loop (cdr ses) last-se se2))
 	    ((not last-se)
-	     (unless (null? ses)
-	       (for-each (lambda (e) (hash-table-set! seen (car e) #t)) se2))
+             (for-each (lambda (e) (hash-table-set! seen (car e) #t)) se2)
 	     (loop ses se2 se2))
 	    (else (let lp ((se (car ses)) (se2 se2))
 		    (cond ((null? se) (loop (cdr ses) (car ses) se2))
@@ -369,14 +376,18 @@
                             (else
                               (let ((name (caar sd)))
                                 (cons `(scheme#cons ',(caar sd) ,(strip-syntax (cdar sd)))
-                                      (loop (cdr sd)))))))))))))))
+                                      (loop (cdr sd)))))))))
+            (scheme#list   ; renames
+              ,@(map (lambda (ren)
+                       `(scheme#cons ',(car ren) ',(cdr ren)))
+                  (module-rename-list mod)))))))))
 
 ;; iexports = indirect exports (syntax dependencies on value idents, explicitly included in module export list)
 ;; vexports = value (non-syntax) exports
 ;; sexports = syntax exports
 ;; sdefs = unexported definitions from syntax environment used by exported macros (not in export list)
 (define (##sys#register-compiled-module name lib iexports vexports sexports #!optional
-					(sdefs '()))
+					(sdefs '()) (renames '()))
   (define (find-reexport name)
     (let ((a (assq name (##sys#macro-environment))))
       (if (and a (pair? (cdr a)))
@@ -396,7 +407,8 @@
 	  (map (lambda (ne)
 		 (list (car ne) #f (##sys#ensure-transformer (cdr ne) (car ne))))
 	       sdefs))
-	 (mod (make-module name lib '() vexports (append sexps reexp-sexps) iexports))
+	 (mod (make-module name lib '() vexports (append sexps reexp-sexps) iexports
+                           renames))
 	 (senv (if (or (not (null? sexps))  ; Only macros have an senv
 		       (not (null? nexps))) ; which must be patched up
 		   (merge-se
@@ -442,9 +454,10 @@
     (set! ##sys#module-table (cons (cons name mod) ##sys#module-table)) 
     mod))
 
-;; same as register-core-module, but uses module's name as its library
+;; same as register-core-module (above) but does not load any code,
+;; used to register modules that provide only syntax
 (define (##sys#register-primitive-module name vexports #!optional (sexports '()))
-  (##sys#register-core-module name name vexports sexports))
+  (##sys#register-core-module name #f vexports sexports))
 
 (define (find-export sym mod indirect)
   (let ((exports (module-export-list mod)))
@@ -521,18 +534,27 @@
 
 	  (##sys#error (get-output-string out))))
 
+      (define (filter-sdlist mod)
+        (let loop ((syms (module-defined-syntax-list mod)))
+          (cond ((null? syms) '())
+                ((eq? (##sys#get (caar syms) '##sys#override) 'value)
+                 (loop (cdr syms)))
+                (else (cons (assq (caar syms) (##sys#macro-environment))
+                            (loop (cdr syms)))))))
+
       (let* ((explist (module-export-list mod))
 	     (name (module-name mod))
 	     (dlist (module-defined-list mod))
 	     (elist (module-exist-list mod))
 	     (missing #f)
-	     (sdlist (map (lambda (sym) (assq (car sym) (##sys#macro-environment)))
-			  (module-defined-syntax-list mod)))
+	     (sdlist (filter-sdlist mod))
 	     (sexports
 	      (if (eq? #t explist)
 		  (merge-se (module-sexports mod) sdlist)
 		  (let loop ((me (##sys#macro-environment)))
 		    (cond ((null? me) '())
+                          ((eq? (##sys#get (caar me) '##sys#override) 'value)
+                           (loop (cdr me)))
 			  ((find-export (caar me) mod #f)
 			   (cons (car me) (loop (cdr me))))
 			  (else (loop (cdr me)))))))
@@ -542,7 +564,9 @@
 		    '()
 		    (let* ((h (car xl))
 			   (id (if (symbol? h) h (car h))))
-		      (cond ((assq id sexports) (loop (cdr xl)))
+		      (cond ((eq? (##sys#get id '##sys#override) 'syntax)
+                              (loop (cdr xl)))
+                            ((assq id sexports) (loop (cdr xl)))
                             (else 
                               (cons 
                                 (cons 
@@ -659,6 +683,16 @@
 	    ((symbol? x) (##sys#symbol->string x))
 	    ((number? x) (number->string x))
 	    (else (##sys#syntax-error-hook loc "invalid prefix" ))))
+    (define (export-rename mod lst)
+      (let ((ren (module-rename-list mod)))
+        (if (null? ren)
+            lst
+            (map (lambda (a)
+                   (cond ((assq (car a) ren) =>
+                          (lambda (b) 
+                            (cons (cdr b) (cdr a))))
+                         (else a)))
+              lst))))
     (call-with-current-continuation
      (lambda (k)
        (define (module-imports name)
@@ -669,10 +703,10 @@
 	       (values (module-name mod)
 		       (module-library mod)
 		       (module-name mod)
-		       (module-vexports mod)
-		       (module-sexports mod)
+		       (export-rename mod (module-vexports mod))
+		       (export-rename mod (module-sexports mod))
 		       (module-iexports mod)))))
-       (let loop ((x x))
+       (let outer ((x x))
 	 (cond ((symbol? x)
 		(module-imports (strip-syntax x)))
 	       ((not (pair? x))
@@ -681,7 +715,7 @@
 		(let ((head (car x)))
 		  (cond ((c %only head)
 			 (##sys#check-syntax loc x '(_ _ . #(symbol 0)))
-			 (let-values (((name lib spec impv imps impi) (loop (cadr x)))
+			 (let-values (((name lib spec impv imps impi) (outer (cadr x)))
 				      ((imports) (strip-syntax (cddr x))))
 			   (let loop ((ids imports) (v '()) (s '()) (missing '()))
 			     (cond ((null? ids)
@@ -700,11 +734,11 @@
 				    (loop (cdr ids) v s (cons (car ids) missing)))))))
 			((c %except head)
 			 (##sys#check-syntax loc x '(_ _ . #(symbol 0)))
-			 (let-values (((name lib spec impv imps impi) (loop (cadr x)))
+			 (let-values (((name lib spec impv imps impi) (outer (cadr x)))
 				      ((imports) (strip-syntax (cddr x))))
-			   (let loop ((impv impv) (v '()) (ids imports))
+			   (let loopv ((impv impv) (v '()) (ids imports))
 			     (cond ((null? impv)
-				    (let loop ((imps imps) (s '()) (ids ids))
+				    (let loops ((imps imps) (s '()) (ids ids))
 				      (cond ((null? imps)
 					     (for-each
 					      (lambda (id)
@@ -713,21 +747,21 @@
 					     (values name lib `(,head ,spec ,@imports) v s impi))
 					    ((memq (caar imps) ids) =>
 								    (lambda (id)
-								      (loop (cdr imps) s (delete (car id) ids eq?))))
+								      (loops (cdr imps) s (delete (car id) ids eq?))))
 					    (else
-					     (loop (cdr imps) (cons (car imps) s) ids)))))
+					     (loops (cdr imps) (cons (car imps) s) ids)))))
 				   ((memq (caar impv) ids) =>
 							   (lambda (id)
-							     (loop (cdr impv) v (delete (car id) ids eq?))))
+							     (loopv (cdr impv) v (delete (car id) ids eq?))))
 				   (else
-				    (loop (cdr impv) (cons (car impv) v) ids))))))
+				    (loopv (cdr impv) (cons (car impv) v) ids))))))
 			((c %rename head)
 			 (##sys#check-syntax loc x '(_ _ . #((symbol symbol) 0)))
-			 (let-values (((name lib spec impv imps impi) (loop (cadr x)))
+			 (let-values (((name lib spec impv imps impi) (outer (cadr x)))
 				      ((renames) (strip-syntax (cddr x))))
-			   (let loop ((impv impv) (v '()) (ids renames))
+			   (let loopv ((impv impv) (v '()) (ids renames))
 			     (cond ((null? impv)
-				    (let loop ((imps imps) (s '()) (ids ids))
+				    (let loops ((imps imps) (s '()) (ids ids))
 				      (cond ((null? imps)
 					     (for-each
 					      (lambda (id)
@@ -736,21 +770,21 @@
 					     (values name lib `(,head ,spec ,@renames) v s impi))
 					    ((assq (caar imps) ids) =>
 					     (lambda (a)
-					       (loop (cdr imps)
+					       (loops (cdr imps)
 						     (cons (cons (cadr a) (cdar imps)) s)
 						     (delete a ids eq?))))
 					    (else
-					     (loop (cdr imps) (cons (car imps) s) ids)))))
+					     (loops (cdr imps) (cons (car imps) s) ids)))))
 				   ((assq (caar impv) ids) =>
 				    (lambda (a)
-				      (loop (cdr impv)
-					    (cons (cons (cadr a) (cdar impv)) v)
-					    (delete a ids eq?))))
+				      (loopv (cdr impv)
+					     (cons (cons (cadr a) (cdar impv)) v)
+					     (delete a ids eq?))))
 				   (else
-				    (loop (cdr impv) (cons (car impv) v) ids))))))
+				    (loopv (cdr impv) (cons (car impv) v) ids))))))
 			((c %prefix head)
 			 (##sys#check-syntax loc x '(_ _ _))
-			 (let-values (((name lib spec impv imps impi) (loop (cadr x)))
+			 (let-values (((name lib spec impv imps impi) (outer (cadr x)))
 				      ((prefix) (strip-syntax (caddr x))))
 			   (define (rename imp)
 			     (cons
@@ -787,17 +821,20 @@
     (dd `(S: ,(if cm (module-name cm) '<toplevel>) ,(map-se vss)))
     (for-each
      (lambda (imp)
-       (and-let* ((id (car imp))
-                  (a (assq id (import-env)))
-                  (aid (cdr imp))
-                  ((not (eq? aid (cdr a)))))
-         (##sys#notice "re-importing already imported identifier" id)))
+       (let ((id (car imp)))
+         (##sys#put! id '##sys#override #f)
+         (and-let* ((a (assq id (import-env)))
+                    (aid (cdr imp))
+                    ((not (eq? aid (cdr a)))))
+              (##sys#notice "re-importing already imported identifier" id))))
      vsv)
     (for-each
      (lambda (imp)
-       (and-let* ((a (assq (car imp) (macro-env)))
-                  ((not (eq? (cdr imp) (cdr a)))))
-         (##sys#notice "re-importing already imported syntax" (car imp))))
+       (let ((id (car imp)))
+         (##sys#put! id '##sys#override #f)
+         (and-let* ((a (assq (car imp) (macro-env)))
+                    ((not (eq? (cdr imp) (cdr a)))))
+              (##sys#notice "re-importing already imported syntax" (car imp)))))
      vss)
     (when reexp?
       (unless cm
@@ -1095,19 +1132,18 @@
   (##sys#register-core-module 'r5rs-null #f '() r4rs-syntax))
 
 (##sys#register-module-alias 'r5rs 'scheme)
-(##sys#register-module-alias 'srfi-88 'chicken.keyword)
 
 (define-inline (se-subset names env) (map (cut assq <> env) names))
 
 ;; Hack for library.scm to use macros from modules it defines itself.
-(##sys#register-core-module
- 'chicken.internal.syntax #f '() (##sys#macro-environment))
+(##sys#register-primitive-module
+ 'chicken.internal.syntax '() (##sys#macro-environment))
 
-(##sys#register-core-module
- 'chicken.module #f '() ##sys#chicken.module-macro-environment)
+(##sys#register-primitive-module
+ 'chicken.module '() ##sys#chicken.module-macro-environment)
 
-(##sys#register-core-module
- 'chicken.type #f '() ##sys#chicken.type-macro-environment)
+(##sys#register-primitive-module
+ 'chicken.type '() ##sys#chicken.type-macro-environment)
 
 (##sys#register-primitive-module
  'srfi-0 '() (se-subset '(cond-expand) ##sys#default-macro-environment))
@@ -1180,8 +1216,8 @@
 (##sys#register-core-module
  'srfi-88 'library
  '((keyword? . chicken.keyword#keyword?)
-   (keyword->string chicken.keyword#keyword->string)
-   (string->keyword chicken.keyword#string->keyword)))
+   (keyword->string . chicken.keyword#keyword->string)
+   (string->keyword . chicken.keyword#string->keyword)))
 
 (##sys#register-core-module
  'srfi-98 'posix
