@@ -1,6 +1,6 @@
 ;;;; posix-common.scm - common code for UNIX and Windows versions of the posix unit
 ;
-; Copyright (c) 2010-2021, The CHICKEN Team
+; Copyright (c) 2010-2022, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -189,7 +189,8 @@ EOF
 	[string-append string-append] )
     (lambda (type loc msg . args)
       (let ([rn (##sys#update-errno)])
-	(apply ##sys#signal-hook type loc (string-append msg " - " (strerror rn)) args) ) ) ) )
+        (apply ##sys#signal-hook/errno
+               type rn loc (string-append msg " - " (strerror rn)) args)))))
 
 (define ##sys#posix-error posix-error)
 
@@ -625,19 +626,46 @@ EOF
 
 ;;; Signals
 
-(set! chicken.process.signal#set-signal-handler!
+(set! chicken.process.signal#set-signal-handler!   ; DEPRECATED
   (lambda (sig proc)
     (##sys#check-fixnum sig 'set-signal-handler!)
     (##core#inline "C_establish_signal_handler" sig (and proc sig))
     (vector-set! ##sys#signal-vector sig proc) ) )
 
-(set! chicken.process.signal#signal-handler
+(set! chicken.process.signal#signal-handler   ; DEPRECATED
   (getter-with-setter
    (lambda (sig)
      (##sys#check-fixnum sig 'signal-handler)
      (##sys#slot ##sys#signal-vector sig) )
    chicken.process.signal#set-signal-handler!
    "(chicken.process.signal#signal-handler sig)"))
+                        
+(set! chicken.process.signal#make-signal-handler
+  (lambda sigs
+    (let ((q (##sys#make-event-queue)))
+      (for-each
+        (lambda (sig)
+          (##sys#check-fixnum sig 'make-signal-handler)
+          (##core#inline "C_establish_signal_handler" sig sig)
+          (vector-set! ##sys#signal-vector sig 
+                       (lambda (sig) (##sys#add-event-to-queue! q sig))))
+        sigs)
+      (lambda (#!optional wait) 
+        (if wait
+            (##sys#wait-for-next-event q)
+            (##sys#get-next-event q))))))
+                        
+(set! chicken.process.signal#signal-ignore
+  (lambda (sig)
+    (##sys#check-fixnum sig 'signal-ignore)
+    (##core#inline "C_establish_signal_handler" sig #f)
+    (vector-set! ##sys#signal-vector sig #f)))
+
+(set! chicken.process.signal#signal-default
+  (lambda (sig)
+    (##sys#check-fixnum sig 'signal-default)
+    (##core#inline "C_establish_signal_handler" sig #t)
+    (vector-set! ##sys#signal-vector sig #f)))
 
 
 ;;; Processes
@@ -709,11 +737,9 @@ EOF
    lst))
 
 (define call-with-exec-args
-  (let ((pathname-strip-directory pathname-strip-directory)
-	(nop (lambda (x) x)))
+  (let ((nop (lambda (x) x)))
     (lambda (loc filename argconv arglist envlist proc)
-      (let* ((stripped-filename (pathname-strip-directory filename))
-	     (args (cons stripped-filename arglist)) ; Add argv[0]
+      (let* ((args (cons filename arglist)) ; Add argv[0]
 	     (argbuf (list->c-string-buffer args argconv loc))
 	     (envbuf #f))
 
