@@ -1,6 +1,6 @@
 ;;;; posixunix.scm - Miscellaneous file- and process-handling routines
 ;
-; Copyright (c) 2008-2021, The CHICKEN Team
+; Copyright (c) 2008-2022, The CHICKEN Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -409,7 +409,6 @@ static int set_file_mtime(char *filename, C_word atime, C_word mtime)
 	   (nfds (fx+ nfdsr nfdsw))
 	   (fds-blob (##sys#make-blob
 		      (fx* nfds (foreign-value "sizeof(struct pollfd)" int)))))
-      (when tm (##sys#check-exact-integer tm))
       (do ((i 0 (fx+ i 1))
 	   (fdsrl fdsrl (cdr fdsrl)))
 	  ((null? fdsrl))
@@ -423,7 +422,7 @@ static int set_file_mtime(char *filename, C_word atime, C_word mtime)
 	   "struct pollfd *fds = p;"
 	   "fds[i].fd = fd; fds[i].events = POLLOUT;") i (car fdswl) fds-blob))
       (let ((n ((foreign-lambda int "poll" scheme-pointer int int)
-		fds-blob nfds (if tm (* (max 0 tm) 1000) -1))))
+		fds-blob nfds (if tm (inexact->exact (truncate (* (max 0 tm) 1000))) -1))))
 	(cond ((fx< n 0)
 	       (posix-error #:file-error 'file-select "failed" fdsr fdsw) )
 	      ((fx= n 0) (values (if (pair? fdsr) '() #f) (if (pair? fdsw) '() #f)))
@@ -604,8 +603,8 @@ static int set_file_mtime(char *filename, C_word atime, C_word mtime)
    (lambda (id)
      (##sys#check-fixnum id 'current-user-id)
      (when (fx< (##core#inline "C_setuid" id) 0)
-       (##sys#update-errno)
-       (##sys#error 'current-user-id!-setter "cannot set user ID" id) ) )
+       (##sys#error/errno (##sys#update-errno)
+                          'current-user-id!-setter "cannot set user ID" id)))
    "(chicken.process-context.posix#current-user-id)"))
 
 (set! chicken.process-context.posix#current-effective-user-id
@@ -614,9 +613,9 @@ static int set_file_mtime(char *filename, C_word atime, C_word mtime)
    (lambda (id)
      (##sys#check-fixnum id 'current-effective-user-id)
      (when (fx< (##core#inline "C_seteuid" id) 0)
-       (##sys#update-errno)
-       (##sys#error
-	'effective-user-id!-setter "cannot set effective user ID" id) ) )
+       (##sys#error/errno (##sys#update-errno)
+                          'effective-user-id!-setter
+                          "cannot set effective user ID" id)))
    "(chicken.process-context.posix#current-effective-user-id)"))
 
 (set! chicken.process-context.posix#current-group-id
@@ -625,8 +624,8 @@ static int set_file_mtime(char *filename, C_word atime, C_word mtime)
    (lambda (id)
      (##sys#check-fixnum id 'current-group-id)
      (when (fx< (##core#inline "C_setgid" id) 0)
-       (##sys#update-errno)
-       (##sys#error 'current-group-id!-setter "cannot set group ID" id) ) )
+       (##sys#error/errno (##sys#update-errno)
+                          'current-group-id!-setter "cannot set group ID" id)))
    "(chicken.process-context.posix#current-group-id)") )
 
 (set! chicken.process-context.posix#current-effective-group-id
@@ -635,9 +634,9 @@ static int set_file_mtime(char *filename, C_word atime, C_word mtime)
    (lambda (id)
      (##sys#check-fixnum id 'current-effective-group-id)
      (when (fx< (##core#inline "C_setegid" id) 0)
-       (##sys#update-errno)
-       (##sys#error
-	'effective-group-id!-setter "cannot set effective group ID" id) ) )
+       (##sys#error/errno (##sys#update-errno)
+                          'effective-group-id!-setter
+                          "cannot set effective group ID" id)))
    "(chicken.process-context.posix#current-effective-group-id)") )
 
 (define-foreign-variable _user-name nonnull-c-string "C_user->pw_name")
@@ -697,8 +696,8 @@ static int set_file_mtime(char *filename, C_word atime, C_word mtime)
   (lambda ()
    (let ([a (##core#inline "C_setsid" #f)])
      (when (fx< a 0)
-	   (##sys#update-errno)
-	   (##sys#error 'create-session "cannot create session") )
+       (##sys#error/errno (##sys#update-errno)
+                          'create-session "cannot create session"))
      a)) )
 
 (set! chicken.process-context.posix#process-group-id
@@ -707,15 +706,16 @@ static int set_file_mtime(char *filename, C_word atime, C_word mtime)
      (##sys#check-fixnum pid 'process-group-id)
      (let ([a (##core#inline "C_getpgid" pid)])
        (when (fx< a 0)
-         (##sys#update-errno)
-         (##sys#error 'process-group-id "cannot retrieve process group ID" pid) )
+         (##sys#error/errno (##sys#update-errno)
+                            'process-group-id
+                            "cannot retrieve process group ID" pid))
        a))
    (lambda (pid pgid)
      (##sys#check-fixnum pid 'process-group)
      (##sys#check-fixnum pgid 'process-group)
      (when (fx< (##core#inline "C_setpgid" pid pgid) 0)
-       (##sys#update-errno)
-       (##sys#error 'process-group "cannot set process group ID" pid pgid) ) )
+       (##sys#error/errno (##sys#update-errno)
+                          'process-group "cannot set process group ID" pid pgid)))
    "(chicken.process-context.posix#process-group-id pid)"))
 
 
@@ -730,7 +730,7 @@ static int set_file_mtime(char *filename, C_word atime, C_word mtime)
               (##sys#make-c-string old 'create-symbolic-link)
               (##sys#make-c-string new 'create-symbolic-link) )
              0)
-      (posix-error #:file-error 'create-symbol-link "cannot create symbolic link" old new) ) ) )
+      (posix-error #:file-error 'create-symbolic-link "cannot create symbolic link" old new) ) ) )
 
 (define-foreign-variable _filename_max int "FILENAME_MAX")
 
