@@ -1,6 +1,6 @@
 ;;;; scrutinizer.scm - The CHICKEN Scheme compiler (local flow analysis)
 ;
-; Copyright (c) 2009-2021, The CHICKEN Team
+; Copyright (c) 2009-2022, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -45,6 +45,7 @@
 	chicken.pathname
 	chicken.platform
 	chicken.plist
+	chicken.sort
 	chicken.port
 	chicken.pretty-print
 	chicken.string
@@ -74,7 +75,7 @@
 ; result specifiers:
 ;
 ;   SPEC = * | (TYPE1 ...)
-;   TYPE = (or TYPE1 ...)
+;   TYPE = (or TYPE1 TYPE2 ...)
 ;        | (not TYPE)
 ;        | (struct NAME)
 ;        | (procedure [NAME] (TYPE1 ... [#!optional TYPE1 ...] [#!rest [TYPE | values]]) . RESULTS)
@@ -87,7 +88,7 @@
 ;        | (deprecated NAME)
 ;   VALUE = string | symbol | keyword | char | number |
 ;           boolean | true | false |
-;           null | eof | blob |  pointer | port | locative | fixnum |
+;           null | eof | bwp | blob |  pointer | port | locative | fixnum |
 ;           float | bignum | ratnum | cplxnum | integer | pointer-vector
 ;   BASIC = * | list | pair | procedure | vector | undefined | noreturn | values
 ;   COMPLEX = (pair TYPE TYPE)
@@ -132,7 +133,7 @@
 (define-constant +maximal-complex-object-constructor-result-type-length+ 256)
 
 (define-constant value-types
-  '(string symbol keyword char null boolean true false blob eof
+  '(string symbol keyword char null boolean true false blob eof bwp
     fixnum float number integer bignum ratnum cplxnum
     pointer-vector port pointer locative))
 
@@ -171,7 +172,7 @@
 	   ((or) (every type-always-immediate? (cdr t)))
 	   ((forall) (type-always-immediate? (third t)))
 	   (else #f)))
-	((memq t '(eof null fixnum char boolean undefined)) #t)
+	((memq t '(eof bwp null fixnum char boolean undefined)) #t)
 	(else #f)))
 
 (define (scrutinize node db complain specialize strict block-compilation)
@@ -212,6 +213,8 @@
 	     (simplify-type
 	      `(pair ,(constant-result (car lit)) ,(constant-result (cdr lit)))))
 	    ((eof-object? lit) 'eof)
+	    ;; TODO: Remove once we have a bootstrapping libchicken with bwp-object?
+	    ((##core#inline "C_bwpp" lit) #;(bwp-object? lit) 'bwp)
 	    ((vector? lit) 
 	     (simplify-type
 	      `(vector ,@(map constant-result (vector->list lit)))))
@@ -1680,42 +1683,60 @@
        (call-with-input-file dbfile read-expressions))
       #t)))
 
+(define (hash-table->list ht)
+  (let ((len (vector-length ht)))
+    (let loop1 ((i 0) (lst '()))
+      (if (>= i len)
+          lst
+          (let loop2 ((bl (vector-ref ht i))
+                      (lst lst))
+            (if (null? bl)
+                (loop1 (add1 i) lst)
+                (loop2 (cdr bl)
+                       (cons (cons (caar bl) (cdar bl)) lst))))))))
+
+(define (symbol<? s1 s2)
+  (string<? (symbol->string s1)
+            (symbol->string s2)))
+
 (define (emit-types-file source-file types-file db block-compilation)
   (with-output-to-file types-file
     (lambda ()
       (print "; GENERATED BY CHICKEN " (chicken-version) " FROM "
 	     source-file "\n")
-      (hash-table-for-each
-       (lambda (sym plist)
-	 (when (and (variable-visible? sym block-compilation)
-		    (memq (variable-mark sym '##compiler#type-source) '(local inference)))
-	   (let ((specs (or (variable-mark sym '##compiler#specializations) '()))
-		 (type (variable-mark sym '##compiler#type))
-		 (pred (variable-mark sym '##compiler#predicate))
-		 (pure (variable-mark sym '##compiler#pure))
-		 (clean (variable-mark sym '##compiler#clean))
-		 (enforce (variable-mark sym '##compiler#enforce))
-		 (foldable (variable-mark sym '##compiler#foldable)))
-	     (pp (cons*
-		  sym
-		  (let wrap ((type type))
-		    (if (pair? type)
-			(case (car type)
-			  ((procedure)
-			   `(#(procedure
-			       ,@(if enforce '(#:enforce) '())
-			       ,@(if pred `(#:predicate ,pred) '())
-			       ,@(if pure '(#:pure) '())
-			       ,@(if clean '(#:clean) '())
-			       ,@(if foldable '(#:foldable) '()))
-			     ,@(cdr type)))
-			  ((forall)
-			   `(forall ,(second type) ,(wrap (third type))))
-			  (else type))
-			type))
-		  specs))
-	     (newline))))
-       db)
+      (for-each
+       (lambda (p)
+         (let ((sym (car p))
+               (plist (cdr p)))
+           (when (and (variable-visible? sym block-compilation)
+                      (memq (variable-mark sym '##compiler#type-source) '(local inference)))
+             (let ((specs (or (variable-mark sym '##compiler#specializations) '()))
+                   (type (variable-mark sym '##compiler#type))
+                   (pred (variable-mark sym '##compiler#predicate))
+                   (pure (variable-mark sym '##compiler#pure))
+                   (clean (variable-mark sym '##compiler#clean))
+                   (enforce (variable-mark sym '##compiler#enforce))
+                   (foldable (variable-mark sym '##compiler#foldable)))
+               (pp (cons* sym
+                          (let wrap ((type type))
+                            (if (pair? type)
+                                (case (car type)
+                                  ((procedure)
+                                   `(#(procedure
+                                                 ,@(if enforce '(#:enforce) '())
+                                                 ,@(if pred `(#:predicate ,pred) '())
+                                                 ,@(if pure '(#:pure) '())
+                                                 ,@(if clean '(#:clean) '())
+                                                 ,@(if foldable '(#:foldable) '()))
+                                                 ,@(cdr type)))
+                                  ((forall)
+                                   `(forall ,(second type) ,(wrap (third type))))
+                                  (else type))
+                                type))
+                          specs))
+               (newline)))))
+       (sort (hash-table->list db)
+             (lambda (a b) (symbol<? (car a) (car b)))))
       (print "; END OF FILE"))))
 
 ;;
@@ -1890,6 +1911,7 @@
 		  v))
 	    ((eq? 'or (car t)) 
 	     (and (list? t)
+		  (not (null? (cdr t)))
 		  (let ((ts (map validate (cdr t))))
 		    (and (every identity ts)
 			 `(or ,@ts)))))
