@@ -1,6 +1,6 @@
 ;;;; support.scm - Miscellaneous support code for the CHICKEN compiler
 ;
-; Copyright (c) 2008-2021, The CHICKEN Team
+; Copyright (c) 2008-2022, The CHICKEN Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -40,9 +40,7 @@
      canonicalize-begin-body string->expr llist-length llist-match?
      expand-profile-lambda reset-profile-info-vector-name!
      profiling-prelude-exps db-get db-get-all db-put! collect! db-get-list
-     get-line get-line-2 display-line-number-database
-     make-node node? node-class node-class-set!
-     node-parameters node-parameters-set!
+     make-node node? node-class node-class-set! node-parameters node-parameters-set!
      node-subexpressions node-subexpressions-set! varnode qnode
      build-node-graph build-expression-tree fold-boolean inline-lambda-bindings
      tree-copy copy-node! copy-node emit-global-inline-file load-inline-file
@@ -58,14 +56,14 @@
      register-foreign-type! lookup-foreign-type clear-foreign-type-table!
      estimate-foreign-result-size estimate-foreign-result-location-size
      finish-foreign-result foreign-type->scrutiny-type scan-used-variables
-     scan-free-variables chop-separator
+     scan-free-variables
      make-block-variable-literal block-variable-literal?
      block-variable-literal-name make-random-name
      clear-real-name-table! get-real-name set-real-name!
      real-name real-name2 display-real-name-table
      source-info->string source-info->line source-info->name
      call-info constant-form-eval maybe-constant-fold-call
-     dump-nodes read-info-hook read/source-info big-fixnum? small-bignum?
+     dump-nodes read/source-info big-fixnum? small-bignum?
      hide-variable export-variable variable-hidden? variable-visible?
      mark-variable variable-mark intrinsic? predicate? foldable?
      load-identifier-database
@@ -289,12 +287,16 @@
 
 ;;; Predicates on expressions and literals:
 
+;; TODO: Remove once we have a bootstrapping libchicken with bwp-object?
+(define (bwp-object? x) (##core#inline "C_bwpp" x))
+
 (define (constant? x)
   (or (number? x)
       (char? x)
       (string? x)
       (boolean? x)
       (eof-object? x)
+      (bwp-object? x)
       (blob? x)
       (vector? x)
       (##sys#srfi-4-vector? x)
@@ -304,6 +306,7 @@
   (or (boolean? x)
       (char? x)
       (eof-object? x)
+      (bwp-object? x)
       (number? x)
       (symbol? x) ) )
 
@@ -312,6 +315,7 @@
       (eq? (##core#undefined) x)
       (null? x)
       (eof-object? x)
+      (bwp-object? x)
       (char? x)
       (boolean? x) ) )
 
@@ -356,7 +360,7 @@
 		      (let loop ((lst '()))
 			(let ((x (read)))
 			  (if (eof-object? x)
-			      (apply values (reverse lst))
+			      (reverse lst)
 			      (loop (cons x lst)))))))))
 	  (cond [(null? xs) '(##core#undefined)]
 		[(null? (cdr xs)) (car xs)]
@@ -442,25 +446,6 @@
     (or x '())))
 
 
-;;; Line-number database management:
-
-(define (get-line exp)
-  (db-get ##sys#line-number-database (car exp) exp) )
-
-(define (get-line-2 exp)
-  (let* ((name (car exp))
-	 (lst (hash-table-ref ##sys#line-number-database name)))
-    (cond ((and lst (assq exp (cdr lst)))
-	   => (lambda (a) (values (car lst) (cdr a))) )
-	  (else (values name #f)) ) ) )
-
-(define (display-line-number-database)
-  (hash-table-for-each
-   (lambda (key val)
-     (when val (printf "~S ~S~%" key (map cdr val))) )
-   ##sys#line-number-database) )
-
-
 ;;; Node creation and -manipulation:
 
 ;; Note: much of this stuff will be overridden by the inline-definitions in "tweaks.scm".
@@ -561,7 +546,7 @@
 	       ((##core#app)
 		(make-node '##core#call (list #t) (map walk (cdr x))) )
 	       (else
-		(receive (name ln) (get-line-2 x)
+		(receive (name ln) (##sys#get-line-2 x)
 		  (make-node
 		   '##core#call
 		   (list (cond [(variable-mark name '##compiler#always-bound-to-procedure)
@@ -1474,18 +1459,9 @@
     (values vars hvars) ) )		; => freevars hiddenvars
 
 
-;;; Some pathname operations:
-
-(define (chop-separator str)		; Used only in batch-driver.scm
-  (let ([len (sub1 (string-length str))])
-    (if (and (> len 0) 
-	     (memq (string-ref str len) '(#\\ #\/)))
-	(substring str 0 len)
-	str) ) )
-
 ;;; Special block-variable literal type:
 
-(define-record-type block-variable-literal 
+(define-record-type block-variable-literal
   (make-block-variable-literal name)
   block-variable-literal?
   (name block-variable-literal-name))	; symbol
@@ -1670,22 +1646,9 @@
   (newline) )
 
 
-;;; Hook for source information
-
-(define (read-info-hook class data val)	; Used here and in compiler.scm
-  (when (and (eq? 'list-info class) (symbol? (car data)))
-    (hash-table-set!
-     ##sys#line-number-database
-     (car data)
-     (alist-cons 
-      data (conc ##sys#current-source-filename ":" val)
-      (or (hash-table-ref ##sys#line-number-database (car data))
-	  '() ) ) ) )
-  data)
-
-(define (read/source-info in)		; Used only in batch-driver
-  (##sys#read in read-info-hook) )
-
+;; DEPRECATED
+(define (read/source-info in)
+  (chicken.syntax#read-with-source-info in) )
 
 ;;; "#> ... <#" syntax:
 
