@@ -46,10 +46,33 @@
 (assert (= 2 (substring-index-ci "o\x00bar" "foo\x00BAR")))
 (assert (not (substring=? "foo\x00a" "foo\x00b" 1 1)))
 (assert (not (substring-ci=? "foo\x00a" "foo\x00b" 1 1)))
+(assert (substring=? "" "" 0 0 0))
+(assert (substring-ci=? "" "" 0 0 0))
+(assert (substring=? "" "abc"))
+(assert (substring-ci=? "" "abc"))
+(assert (substring=? "abc" ""))
+(assert (substring-ci=? "abc" ""))
+(assert-error (substring=? "" "" 0 0 1))
+(assert-error (substring-ci=? "" "" 0 0 1))
+(assert (substring=? "foo" "foo" 0 0 3))
+(assert (substring-ci=? "foo" "foo" 0 0 3))
 (assert (not (substring-index "o\x00bar" "foo\x00baz")))
 (assert (not (substring-index-ci "o\x00bar" "foo\x00baz")))
 (assert (= 0 (substring-index "" "")))
 (assert (= 1 (substring-index "" "a" 1)))
+(assert-error (substring=? "a" "a" 2))
+(assert-error (substring=? "a" "a" -2))
+(assert-error (substring=? "a" "a" 0 2))
+(assert-error (substring=? "a" "a" 0 -2))
+(assert-error (substring=? "a" "a" 0 0 2))
+(assert-error (substring=? "a" "a" 0 0 -2))
+(assert-error (substring-ci=? "a" "a" 2))
+(assert-error (substring-ci=? "a" "a" -2))
+(assert-error (substring-ci=? "a" "a" 0 2))
+(assert-error (substring-ci=? "a" "a" 0 -2))
+(assert-error (substring-ci=? "a" "a" 0 0 2))
+(assert-error (substring-ci=? "a" "a" 0 0 -2))
+(assert-error (substring-ci=? "a" "a" 0 0 2))
 (assert-error (substring-index "" "a" 2))
 (assert-error (substring-index "a" "b" 2))
 (assert (not (substring-index "a" "b" 1)))
