@@ -380,6 +380,41 @@
    (import (scheme) (chicken module))
    (eq? (current-module) 'm33)))
 
+(module m34 ((syn bar) alias)
+  (import scheme (chicken base) (chicken module))
+  (export/rename (bar baz) (syn syn2))
+  (define bar 123)
+  (assert (equal? bar 123))
+  (define-syntax alias 
+    (syntax-rules () 
+      ((_) (syn))))
+  (define-syntax syn
+    (syntax-rules ()
+      ((_) (list bar)))))
+
+(module m35 ()
+  (import scheme (chicken base) (chicken module))
+  (import (only (rename m34 (syn2 syn3)) syn3 alias))
+  (import (rename m34 (baz bax)))
+  (define bar 99)
+  (assert (equal? bax 123))
+  (assert (equal? (syn3) '(123)))
+  (assert (equal? (alias) '(123)))
+  (assert (equal? bar 99)))
+
+;; corner case, found by DeeEff, actually not really a good idea,
+;; but the expander looped here endlessly
+(module m36 (xcons)
+  (import scheme)
+  (define (xcons x y) (cons y x)))
+  
+(module m37 ()
+  (import (rename m36
+                  (xcons m36#xcons)))
+  (import scheme (chicken base))
+  (define (xcons x y) (m36#xcons 'X x))
+  (assert (equal? '(1 . X) (xcons 1 2))))
+
 (test-end "modules")
 
 (test-exit)
