@@ -63,3 +63,21 @@ freef(void *r)
   (print "forcing remaining")
   (##sys#force-finalizers)
   (assert (= *n* *count*)))
+
+;;; new finalizer API
+
+(define c1 (list *count*))
+(define f1 (make-finalizer c1))
+(add-to-finalizer f1 (make-vector 10))
+(define f2 (make-finalizer f1))
+(gc #t)
+(assert (vector? (f1)))
+(assert (not (f1)))
+(set! c1 #f)
+(gc #t)
+(assert (equal? (f1) (list *count*)))
+(assert (not (f1)))
+(set! f1 #f)
+(gc #t)
+(assert (procedure? (f2)))
+(assert (not (f2)))
